@@ -9,35 +9,27 @@
 #
 # Environment variables:
 #
+#   AUDITWHEEL_PLAT           - Linux platform for auditwheel repair
 #   LIBSSH2_OPENSSL           - Where to find openssl
-#   LIBSSH2_PREFIX            - Where to find libssh2
 #   LIBSSH2_VERSION=<Version> - Build the given version of libssh2
 #   LIBGIT2_VERSION=<Version> - Build the given version of libgit2
 #   OPENSSL_VERSION=<Version> - Build the given version of OpenSSL
 #                               (only needed for Mac universal on CI)
 #
-# Either use LIBSSH2_PREFIX, or LIBSSH2_VERSION, or none (if libssh2 is already
-# in the path, or if you don't want to use it).
-#
 # Examples.
 #
 # Build inplace, libgit2 must be available in the path:
 #
 #   sh build.sh
 #
-# Build libgit2 1.5.0 (will use libssh2 if available), then build pygit2
+# Build libgit2 1.7.2 (will use libssh2 if available), then build pygit2
 # inplace:
 #
-#   LIBGIT2_VERSION=1.5.0 sh build.sh
+#   LIBGIT2_VERSION=1.7.2 sh build.sh
 #
-# Build libssh2 1.10.0 and libgit2 1.5.0, then build pygit2 inplace:
+# Build libssh2 1.11.0 and libgit2 1.7.2, then build pygit2 inplace:
 #
-#   LIBSSH2_VERSION=1.10.0 LIBGIT2_VERSION=1.5.0 sh build.sh
-#
-# Tell where libssh2 is installed, build libgit2 1.5.0, then build pygit2
-# inplace:
-#
-#   LIBSSH2_PREFIX=/usr/local LIBGIT2_VERSION=1.5.0 sh build.sh
+#   LIBSSH2_VERSION=1.11.0 LIBGIT2_VERSION=1.7.2 sh build.sh
 #
 # Build inplace and run the tests:
 #
@@ -65,18 +57,32 @@ PREFIX="${PREFIX:-$(pwd)/ci/$PYTHON_TAG}"
 export LDFLAGS="-Wl,-rpath,$PREFIX/lib"
 
 if [ "$CIBUILDWHEEL" = "1" ]; then
+    if [ -f /usr/bin/apt-get ]; then
+        apt-get update
+        apt-get install wget -y
+        if [ -z "$OPENSSL_VERSION" ]; then
+            apt-get install libssl-dev -y
+        fi
+    elif [ -f /usr/bin/yum ]; then
+        yum install wget zlib-devel -y
+        if [ -z "$OPENSSL_VERSION" ]; then
+            yum install openssl-devel -y
+        else
+            yum install perl-IPC-Cmd -y
+        fi
+    elif [ -f /sbin/apk ]; then
+        apk add wget
+        if [ -z "$OPENSSL_VERSION" ]; then
+            apk add openssl-dev
+        fi
+    fi
     rm -rf ci
     mkdir ci || true
     cd ci
-    if [ -f /usr/bin/apt-get ]; then
-        apt-get update
-        apt-get install libssl-dev wget -y
-    elif [ -f /usr/bin/yum ]; then
-        yum install wget openssl-devel zlib-devel -y
-    fi
 else
     # Create a virtual environment
     $PYTHON -m venv $PREFIX
+    cd ci
 fi
 
 # Install zlib
@@ -94,40 +100,45 @@ fi
 
 # Install openssl
 if [ -n "$OPENSSL_VERSION" ]; then
-    if [ "$CIBUILDWHEEL" != "1" ] || [ "$KERNEL" != "Darwin" ]; then
-        echo "OPENSSL_VERSION should only be set when building"
-        echo "macOS universal2 wheels on GitHub!"
-        echo "Please unset and try again"
-        exit 1
-    fi
     FILENAME=openssl-$OPENSSL_VERSION
     wget https://www.openssl.org/source/$FILENAME.tar.gz -N --no-check-certificate
 
-    tar xf $FILENAME.tar.gz
-    mv $FILENAME openssl-x86
+    if [ "$KERNEL" = "Darwin" ]; then
+        tar xf $FILENAME.tar.gz
+        mv $FILENAME openssl-x86
 
-    tar xf $FILENAME.tar.gz
-    mv $FILENAME openssl-arm
+        tar xf $FILENAME.tar.gz
+        mv $FILENAME openssl-arm
 
-    cd openssl-x86
-    ./Configure darwin64-x86_64-cc shared
-    make
-    cd ../openssl-arm
-    ./Configure enable-rc5 zlib darwin64-arm64-cc no-asm
-    make
-    cd ..
+        cd openssl-x86
+        ./Configure darwin64-x86_64-cc shared
+        make
+        cd ../openssl-arm
+        ./Configure enable-rc5 zlib darwin64-arm64-cc no-asm
+        make
+        cd ..
 
-    mkdir openssl-universal
+        mkdir openssl-universal
 
-    LIBSSL=$(basename openssl-x86/libssl.*.dylib)
-    lipo -create openssl-x86/libssl.*.dylib openssl-arm/libssl.*.dylib -output openssl-universal/$LIBSSL
-    LIBCRYPTO=$(basename openssl-x86/libcrypto.*.dylib)
-    lipo -create openssl-x86/libcrypto.*.dylib openssl-arm/libcrypto.*.dylib -output openssl-universal/$LIBCRYPTO
-    cd openssl-universal
-    install_name_tool -id "@rpath/$LIBSSL" $LIBSSL
-    install_name_tool -id "@rpath/$LIBCRYPTO" $LIBCRYPTO
-    OPENSSL_PREFIX=$(pwd)
-    cd ..
+        LIBSSL=$(basename openssl-x86/libssl.*.dylib)
+        lipo -create openssl-x86/libssl.*.dylib openssl-arm/libssl.*.dylib -output openssl-universal/$LIBSSL
+        LIBCRYPTO=$(basename openssl-x86/libcrypto.*.dylib)
+        lipo -create openssl-x86/libcrypto.*.dylib openssl-arm/libcrypto.*.dylib -output openssl-universal/$LIBCRYPTO
+        cd openssl-universal
+        install_name_tool -id "@rpath/$LIBSSL" $LIBSSL
+        install_name_tool -id "@rpath/$LIBCRYPTO" $LIBCRYPTO
+        OPENSSL_PREFIX=$(pwd)
+        cd ..
+    else
+        # Linux
+        tar xf $FILENAME.tar.gz
+        cd $FILENAME
+        ./Configure shared --prefix=$PREFIX --libdir=$PREFIX/lib
+        make
+        make install
+        OPENSSL_PREFIX=$(pwd)
+        cd ..
+    fi
 fi
 
 # Install libssh2
@@ -155,7 +166,6 @@ if [ -n "$LIBSSH2_VERSION" ]; then
     fi
     cmake --build . --target install
     cd ..
-    LIBSSH2_PREFIX=$PREFIX
     USE_SSH=ON
 else
     USE_SSH=OFF
@@ -167,8 +177,10 @@ if [ -n "$LIBGIT2_VERSION" ]; then
     wget https://github.com/libgit2/libgit2/archive/refs/tags/v$LIBGIT2_VERSION.tar.gz -N -O $FILENAME.tar.gz
     tar xf $FILENAME.tar.gz
     cd $FILENAME
+    mkdir build -p
+    cd build
     if [ "$KERNEL" = "Darwin" ] && [ "$CIBUILDWHEEL" = "1" ]; then
-        CMAKE_PREFIX_PATH=$OPENSSL_PREFIX:$LIBSSH2_PREFIX cmake . \
+        CMAKE_PREFIX_PATH=$OPENSSL_PREFIX:$PREFIX cmake .. \
                 -DBUILD_SHARED_LIBS=ON \
                 -DBUILD_TESTS=OFF \
                 -DCMAKE_BUILD_TYPE=$BUILD_TYPE \
@@ -176,9 +188,11 @@ if [ -n "$LIBGIT2_VERSION" ]; then
                 -DOPENSSL_CRYPTO_LIBRARY="../openssl-universal/$LIBCRYPTO" \
                 -DOPENSSL_SSL_LIBRARY="../openssl-universal/$LIBSSL" \
                 -DOPENSSL_INCLUDE_DIR="../openssl-x86/include" \
+                -DCMAKE_INSTALL_PREFIX=$PREFIX \
                 -DUSE_SSH=$USE_SSH
     else
-        CMAKE_PREFIX_PATH=$OPENSSL_PREFIX:$LIBSSH2_PREFIX cmake . \
+        export CFLAGS=-I$PREFIX/include
+        CMAKE_PREFIX_PATH=$OPENSSL_PREFIX:$PREFIX cmake .. \
                 -DBUILD_SHARED_LIBS=ON \
                 -DBUILD_TESTS=OFF \
                 -DCMAKE_BUILD_TYPE=$BUILD_TYPE \
@@ -187,23 +201,25 @@ if [ -n "$LIBGIT2_VERSION" ]; then
     fi
     cmake --build . --target install
     cd ..
+    cd ..
     export LIBGIT2=$PREFIX
 fi
 
 if [ "$CIBUILDWHEEL" = "1" ]; then
     if [ "$KERNEL" = "Darwin" ]; then
-        # Copy libraries where delocate-wheel can find them.
-        # In Linux we use LD_LIBRARY_PATH to avoid this, maybe
-        # DYLD_LIBRARY_PATH would work for macOS.
-        cp -r $OPENSSL_PREFIX/*.dylib /usr/local/lib
-        cp -r $LIBSSH2_PREFIX/lib/*.dylib /usr/local/lib
-        cp -r $FILENAME/*.dylib /usr/local/lib
+        cp $OPENSSL_PREFIX/*.dylib $PREFIX/lib/
+        cp $OPENSSL_PREFIX/*.dylib $PREFIX/lib/
+        echo "PREFIX        " $PREFIX
+        echo "OPENSSL_PREFIX" $OPENSSL_PREFIX
+        ls -l /Users/runner/work/pygit2/pygit2/ci/
+        ls -l $PREFIX/lib
     fi
     # we're done building dependencies, cibuildwheel action will take over
     exit 0
 fi
 
 # Build pygit2
+cd ..
 $PREFIX/bin/pip install -U pip wheel
 if [ "$1" = "wheel" ]; then
     shift
@@ -252,4 +268,3 @@ if [ "$1" = "stubtest" ]; then
     PYTHONPATH=. $PREFIX/bin/stubtest --mypy-config-file mypy-stubtest.ini pygit2._pygit2
     [ $? == 0 ] && echo "stubtest OK"
 fi
-
