@@ -1,4 +1,4 @@
-# Copyright 2010-2019 The pygit2 contributors
+# Copyright 2010-2024 The pygit2 contributors
 #
 # This file is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License, version 2,
@@ -26,7 +26,7 @@
 # Import from pygit2
 from .ffi import ffi, C
 from .utils import GenericIterator
-from _pygit2 import Signature, Oid
+from ._pygit2 import Signature, Oid
 
 
 def wrap_signature(csig):
@@ -37,7 +37,7 @@ def wrap_signature(csig):
                      ffi.string(csig.email).decode('utf-8'),
                      csig.when.time, csig.when.offset, 'utf-8')
 
-class BlameHunk(object):
+class BlameHunk:
 
     @classmethod
     def _from_c(cls, blame, ptr):
@@ -95,7 +95,7 @@ class BlameHunk(object):
         return ffi.string(path).decode('utf-8')
 
 
-class Blame(object):
+class Blame:
 
     @classmethod
     def _from_c(cls, repo, ptr):
