@@ -1,5 +1,5 @@
 /*
- * Copyright 2010-2021 The pygit2 contributors
+ * Copyright 2010-2024 The pygit2 contributors
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2,
@@ -47,6 +47,10 @@ extern PyTypeObject DiffLineType;
 extern PyTypeObject DiffStatsType;
 extern PyTypeObject RepositoryType;
 
+extern PyObject *DeltaStatusEnum;
+extern PyObject *DiffFlagEnum;
+extern PyObject *FileModeEnum;
+
 PyObject *
 wrap_diff(git_diff *diff, Repository *repo)
 {
@@ -204,13 +208,31 @@ DiffFile_from_c(DiffFile *dummy, PyObject *py_diff_file_ptr)
     return wrap_diff_file(diff_file);
 }
 
+PyDoc_STRVAR(DiffFile_flags__doc__,
+    "A combination of enums.DiffFlag constants."
+);
+
+PyObject *
+DiffFile_flags__get__(DiffFile *self)
+{
+    return pygit2_enum(DiffFlagEnum, self->flags);
+}
+
+PyDoc_STRVAR(DiffFile_mode__doc__,
+    "Mode of the entry (an enums.FileMode constant)."
+);
+
+PyObject *
+DiffFile_mode__get__(DiffFile *self)
+{
+    return pygit2_enum(FileModeEnum, self->mode);
+}
+
 PyMemberDef DiffFile_members[] = {
     MEMBER(DiffFile, id, T_OBJECT, "Oid of the item."),
     MEMBER(DiffFile, path, T_STRING, "Path to the entry."),
     MEMBER(DiffFile, raw_path, T_OBJECT, "Path to the entry (bytes)."),
     MEMBER(DiffFile, size, T_LONG, "Size of the entry."),
-    MEMBER(DiffFile, flags, T_UINT, "Combination of GIT_DIFF_FLAG_* flags."),
-    MEMBER(DiffFile, mode, T_USHORT, "Mode of the entry."),
     {NULL}
 };
 
@@ -219,6 +241,12 @@ PyMethodDef DiffFile_methods[] = {
     {NULL},
 };
 
+PyGetSetDef DiffFile_getsetters[] = {
+    GETTER(DiffFile, flags),
+    GETTER(DiffFile, mode),
+    {NULL},
+};
+
 PyDoc_STRVAR(DiffFile__doc__, "DiffFile object.");
 
 PyTypeObject DiffFileType = {
@@ -251,7 +279,7 @@ PyTypeObject DiffFileType = {
     0,                                         /* tp_iternext       */
     DiffFile_methods,                          /* tp_methods        */
     DiffFile_members,                          /* tp_members        */
-    0,                                         /* tp_getset         */
+    DiffFile_getsetters,                       /* tp_getset         */
     0,                                         /* tp_base           */
     0,                                         /* tp_dict           */
     0,                                         /* tp_descr_get      */
@@ -294,6 +322,26 @@ DiffDelta_is_binary__get__(DiffDelta *self)
     Py_RETURN_NONE;
 }
 
+PyDoc_STRVAR(DiffDelta_status__doc__,
+    "An enums.FileStatus constant."
+);
+
+PyObject *
+DiffDelta_status__get__(DiffDelta *self)
+{
+    return pygit2_enum(DeltaStatusEnum, self->status);
+}
+
+PyDoc_STRVAR(DiffDelta_flags__doc__,
+    "A combination of enums.DiffFlag constants."
+);
+
+PyObject *
+DiffDelta_flags__get__(DiffDelta *self)
+{
+    return pygit2_enum(DiffFlagEnum, self->flags);
+}
+
 static void
 DiffDelta_dealloc(DiffDelta *self)
 {
@@ -308,8 +356,6 @@ static PyMethodDef DiffDelta_methods[] = {
 };
 
 PyMemberDef DiffDelta_members[] = {
-    MEMBER(DiffDelta, status, T_UINT, "A GIT_DELTA_* constant."),
-    MEMBER(DiffDelta, flags, T_UINT, "Combination of GIT_DIFF_FLAG_* flags."),
     MEMBER(DiffDelta, similarity, T_USHORT, "For renamed and copied."),
     MEMBER(DiffDelta, nfiles, T_USHORT, "Number of files in the delta."),
     MEMBER(DiffDelta, old_file, T_OBJECT, "\"from\" side of the diff."),
@@ -319,6 +365,8 @@ PyMemberDef DiffDelta_members[] = {
 
 PyGetSetDef DiffDelta_getsetters[] = {
     GETTER(DiffDelta, is_binary),
+    GETTER(DiffDelta, status),
+    GETTER(DiffDelta, flags),
     {NULL}
 };
 
@@ -813,7 +861,7 @@ DiffStats_files_changed__get__(DiffStats *self)
 }
 
 PyDoc_STRVAR(DiffStats_format__doc__,
-    "format(format: int, width: int) -> str\n"
+    "format(format: enums.DiffStatsFormat, width: int) -> str\n"
     "\n"
     "Format the stats as a string.\n"
     "\n"
@@ -822,7 +870,7 @@ PyDoc_STRVAR(DiffStats_format__doc__,
     "Parameters:\n"
     "\n"
     "format\n"
-    "    The format to use. A pygit2.GIT_DIFF_STATS_* constant.\n"
+    "    The format to use. A combination of DiffStatsFormat constants.\n"
     "\n"
     "width\n"
     "    The width of the output. The output will be scaled to fit.");
@@ -967,7 +1015,7 @@ Diff_merge(Diff *self, PyObject *args)
 
 
 PyDoc_STRVAR(Diff_find_similar__doc__,
-  "find_similar(flags: int = GIT_DIFF_FIND_BY_CONFIG, rename_threshold: int = 50, copy_threshold: int = 50, rename_from_rewrite_threshold: int = 50, break_rewrite_threshold: int = 60, rename_limit: int = 1000)\n"
+  "find_similar(flags: enums.DiffFind = enums.DiffFind.FIND_BY_CONFIG, rename_threshold: int = 50, copy_threshold: int = 50, rename_from_rewrite_threshold: int = 50, break_rewrite_threshold: int = 60, rename_limit: int = 1000)\n"
   "\n"
   "Transform a diff marking file renames, copies, etc.\n"
   "\n"
@@ -976,7 +1024,7 @@ PyDoc_STRVAR(Diff_find_similar__doc__,
   "will, if requested, break modified files into add/remove pairs if the "
   "amount of change is above a threshold.\n"
   "\n"
-  "flags - Combination of GIT_DIFF_FIND_* and GIT_DIFF_BREAK_* constants."
+  "flags - Combination of enums.DiffFind.FIND_* and enums.DiffFind.BREAK_* constants."
   );
 
 PyObject *
