@@ -1,5 +1,5 @@
 /*
- * Copyright 2010-2021 The pygit2 contributors
+ * Copyright 2010-2024 The pygit2 contributors
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2,
@@ -40,6 +40,7 @@ extern PyTypeObject TreeType;
 extern PyTypeObject CommitType;
 extern PyTypeObject BlobType;
 extern PyTypeObject TagType;
+extern PyObject *FileModeEnum;
 
 PyTypeObject ObjectType;
 
@@ -132,8 +133,7 @@ Object_short_id__get__(Object *self)
 
 
 PyDoc_STRVAR(Object_type__doc__,
-    "One of the GIT_OBJ_COMMIT, GIT_OBJ_TREE, GIT_OBJ_BLOB or GIT_OBJ_TAG\n"
-    "constants.");
+    "One of the enums.ObjectType.COMMIT, TREE, BLOB or TAG constants.");
 
 PyObject *
 Object_type__get__(Object *self)
@@ -160,7 +160,7 @@ Object__pointer__get__(Object *self)
 }
 
 PyDoc_STRVAR(Object_name__doc__,
-             "Name (will be None if the object was not reached trough a tree)");
+             "Name (or None if the object was not reached through a tree)");
 PyObject *
 Object_name__get__(Object *self)
 {
@@ -182,14 +182,14 @@ Object_raw_name__get__(Object *self)
 }
 
 PyDoc_STRVAR(Object_filemode__doc__,
-             "Filemode (will be None if the object was not reached trough a tree)");
+             "An enums.FileMode constant (or None if the object was not reached through a tree)");
 PyObject *
 Object_filemode__get__(Object *self)
 {
     if (self->entry == NULL)
         Py_RETURN_NONE;
 
-    return PyLong_FromLong(git_tree_entry_filemode(self->entry));
+    return pygit2_enum(FileModeEnum, git_tree_entry_filemode(self->entry));
 }
 
 
@@ -243,7 +243,7 @@ Object_peel(Object *self, PyObject *py_type)
     if (Object__load(self) == NULL) { return NULL; } // Lazy load
 
     otype = py_object_to_otype(py_type);
-    if (otype == GIT_OBJ_BAD)
+    if (otype == GIT_OBJECT_INVALID)
         return NULL;
 
     err = git_object_peel(&peeled, self->obj, otype);
@@ -383,16 +383,16 @@ wrap_object(git_object *c_object, Repository *repo, const git_tree_entry *entry)
     git_object_t obj_type = (c_object) ? git_object_type(c_object) : git_tree_entry_type(entry);
 
     switch (obj_type) {
-        case GIT_OBJ_COMMIT:
+        case GIT_OBJECT_COMMIT:
             py_obj = PyObject_New(Object, &CommitType);
             break;
-        case GIT_OBJ_TREE:
+        case GIT_OBJECT_TREE:
             py_obj = PyObject_New(Object, &TreeType);
             break;
-        case GIT_OBJ_BLOB:
+        case GIT_OBJECT_BLOB:
             py_obj = PyObject_New(Object, &BlobType);
             break;
-        case GIT_OBJ_TAG:
+        case GIT_OBJECT_TAG:
             py_obj = PyObject_New(Object, &TagType);
             break;
         default:
