@@ -1,5 +1,5 @@
 /*
- * Copyright 2010-2021 The pygit2 contributors
+ * Copyright 2010-2024 The pygit2 contributors
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2,
@@ -29,17 +29,27 @@
 #include <Python.h>
 
 #include <git2.h>
+#include <git2/sys/filter.h>
 #include "error.h"
 #include "types.h"
 #include "utils.h"
 #include "repository.h"
 #include "oid.h"
 #include "options.h"
+#include "filter.h"
 
 PyObject *GitError;
 PyObject *AlreadyExistsError;
 PyObject *InvalidSpecError;
 
+PyObject *DeltaStatusEnum;
+PyObject *DiffFlagEnum;
+PyObject *FileModeEnum;
+PyObject *FileStatusEnum;
+PyObject *MergeAnalysisEnum;
+PyObject *MergePreferenceEnum;
+PyObject *ReferenceTypeEnum;
+
 extern PyTypeObject RepositoryType;
 extern PyTypeObject OdbType;
 extern PyTypeObject OdbBackendType;
@@ -79,6 +89,8 @@ extern PyTypeObject NoteIterType;
 extern PyTypeObject WorktreeType;
 extern PyTypeObject MailmapType;
 extern PyTypeObject StashType;
+extern PyTypeObject RefsIteratorType;
+extern PyTypeObject FilterSourceType;
 
 
 PyDoc_STRVAR(discover_repository__doc__,
@@ -143,7 +155,7 @@ hashfile(PyObject *self, PyObject *args)
     if (py_path != NULL)
         path = PyBytes_AS_STRING(py_path);
 
-    err = git_odb_hashfile(&oid, path, GIT_OBJ_BLOB);
+    err = git_odb_hashfile(&oid, path, GIT_OBJECT_BLOB);
     Py_XDECREF(py_path);
     if (err < 0)
         return Error_set(err);
@@ -167,7 +179,7 @@ hash(PyObject *self, PyObject *args)
     if (!PyArg_ParseTuple(args, "s#", &data, &size))
         return NULL;
 
-    err = git_odb_hash(&oid, data, size, GIT_OBJ_BLOB);
+    err = git_odb_hash(&oid, data, size, GIT_OBJECT_BLOB);
     if (err < 0) {
         return Error_set(err);
     }
@@ -262,6 +274,163 @@ tree_entry_cmp(PyObject *self, PyObject *args)
     return PyLong_FromLong(cmp);
 }
 
+PyDoc_STRVAR(filter_register__doc__,
+    "filter_register(name: str, filter_cls: Type[Filter], [priority: int = C.GIT_FILTER_DRIVER_PRIORITY]) -> None\n"
+    "\n"
+    "Register a filter under the given name.\n"
+    "\n"
+    "Filters will be run in order of `priority` on smudge (to workdir) and in\n"
+    "reverse order of priority on clean (to odb).\n"
+    "\n"
+    "Two filters are preregistered with libgit2:\n"
+    "    - GIT_FILTER_CRLF with priority 0\n"
+    "    - GIT_FILTER_IDENT with priority 100\n"
+    "\n"
+    "`priority` defaults to GIT_FILTER_DRIVER_PRIORITY which imitates a core\n"
+    "Git filter driver that will be run last on checkout (smudge) and first \n"
+    "on checkin (clean).\n"
+    "\n"
+    "Note that the filter registry is not thread safe. Any registering or\n"
+    "deregistering of filters should be done outside of any possible usage\n"
+    "of the filters.\n");
+
+PyObject *
+filter_register(PyObject *self, PyObject *args, PyObject *kwds)
+{
+    const char *name;
+    Py_ssize_t size;
+    PyObject *py_filter_cls;
+    int priority = GIT_FILTER_DRIVER_PRIORITY;
+    char *keywords[] = {"name", "filter_cls", "priority", NULL};
+    pygit2_filter *filter;
+    PyObject *py_attrs;
+    PyObject *result = Py_None;
+    int err;
+
+    if (!PyArg_ParseTupleAndKeywords(args, kwds, "s#O|i", keywords,
+                                     &name, &size, &py_filter_cls, &priority))
+        return NULL;
+
+    py_attrs = PyObject_GetAttrString(py_filter_cls, "attributes");
+    if (py_attrs == NULL)
+        return NULL;
+
+    filter = malloc(sizeof(pygit2_filter));
+    if (filter == NULL)
+    {
+        return PyExc_MemoryError;
+    }
+    memset(filter, 0, sizeof(pygit2_filter));
+    git_filter_init(&filter->filter, GIT_FILTER_VERSION);
+
+    filter->filter.attributes = PyUnicode_AsUTF8(py_attrs);
+    filter->filter.shutdown = pygit2_filter_shutdown;
+    filter->filter.check = pygit2_filter_check;
+    filter->filter.stream = pygit2_filter_stream;
+    filter->filter.cleanup = pygit2_filter_cleanup;
+    filter->py_filter_cls = py_filter_cls;
+    Py_INCREF(py_filter_cls);
+
+    if ((err = git_filter_register(name, &filter->filter, priority)) < 0)
+        goto error;
+
+    goto done;
+
+error:
+    Py_DECREF(py_filter_cls);
+    free(filter);
+done:
+    Py_DECREF(py_attrs);
+    return result;
+}
+
+PyDoc_STRVAR(filter_unregister__doc__,
+    "filter_unregister(name: str) -> None\n"
+    "\n"
+    "Unregister the given filter.\n"
+    "\n"
+    "Note that the filter registry is not thread safe. Any registering or\n"
+    "deregistering of filters should be done outside of any possible usage\n"
+    "of the filters.\n");
+
+PyObject *
+filter_unregister(PyObject *self, PyObject *args)
+{
+    const char *name;
+    Py_ssize_t size;
+    int err;
+
+    if (!PyArg_ParseTuple(args, "s#", &name, &size))
+        return NULL;
+    if ((err = git_filter_unregister(name)) < 0)
+        return Error_set(err);
+
+    Py_RETURN_NONE;
+}
+
+
+static void
+forget_enums(void)
+{
+    Py_CLEAR(DeltaStatusEnum);
+    Py_CLEAR(DiffFlagEnum);
+    Py_CLEAR(FileModeEnum);
+    Py_CLEAR(FileStatusEnum);
+    Py_CLEAR(MergeAnalysisEnum);
+    Py_CLEAR(MergePreferenceEnum);
+    Py_CLEAR(ReferenceTypeEnum);
+}
+
+PyDoc_STRVAR(_cache_enums__doc__,
+    "_cache_enums()\n"
+    "\n"
+    "For internal use only. Do not call this from user code.\n"
+    "\n"
+    "Let the _pygit2 C module cache references to Python enums\n"
+    "defined in pygit2.enums.\n");
+
+PyObject *
+_cache_enums(PyObject *self, PyObject *args)
+{
+    (void) args;
+
+    /* In case this is somehow being called several times, let go of old references */
+    forget_enums();
+
+    PyObject *enums = PyImport_ImportModule("pygit2.enums");
+    if (enums == NULL) {
+        return NULL;
+    }
+
+#define CACHE_PYGIT2_ENUM(name) do { \
+    name##Enum = PyObject_GetAttrString(enums, #name); \
+    if (name##Enum == NULL) { goto fail; } \
+} while (0)
+
+    CACHE_PYGIT2_ENUM(DeltaStatus);
+    CACHE_PYGIT2_ENUM(DiffFlag);
+    CACHE_PYGIT2_ENUM(FileMode);
+    CACHE_PYGIT2_ENUM(FileStatus);
+    CACHE_PYGIT2_ENUM(MergeAnalysis);
+    CACHE_PYGIT2_ENUM(MergePreference);
+    CACHE_PYGIT2_ENUM(ReferenceType);
+
+#undef CACHE_PYGIT2_ENUM
+
+    Py_RETURN_NONE;
+
+fail:
+    Py_DECREF(enums);
+    forget_enums();
+    return NULL;
+}
+
+void
+free_module(void *self)
+{
+    forget_enums();
+}
+
 
 PyMethodDef module_methods[] = {
     {"discover_repository", discover_repository, METH_VARARGS, discover_repository__doc__},
@@ -271,10 +440,12 @@ PyMethodDef module_methods[] = {
     {"option", option, METH_VARARGS, option__doc__},
     {"reference_is_valid_name", reference_is_valid_name, METH_O, reference_is_valid_name__doc__},
     {"tree_entry_cmp", tree_entry_cmp, METH_VARARGS, tree_entry_cmp__doc__},
+    {"filter_register", (PyCFunction)filter_register, METH_VARARGS | METH_KEYWORDS, filter_register__doc__},
+    {"filter_unregister", filter_unregister, METH_VARARGS, filter_unregister__doc__},
+    {"_cache_enums", _cache_enums, METH_NOARGS, _cache_enums__doc__},
     {NULL}
 };
 
-
 struct PyModuleDef moduledef = {
     PyModuleDef_HEAD_INIT,
     "_pygit2",                       /* m_name */
@@ -284,7 +455,7 @@ struct PyModuleDef moduledef = {
     NULL,                            /* m_reload */
     NULL,                            /* m_traverse */
     NULL,                            /* m_clear */
-    NULL,                            /* m_free */
+    free_module,                     /* m_free */
 };
 
 PyMODINIT_FUNC
@@ -371,6 +542,7 @@ PyInit__pygit2(void)
     INIT_TYPE(TreeBuilderType, NULL, NULL)
     INIT_TYPE(BlobType, &ObjectType, NULL)
     INIT_TYPE(TagType, &ObjectType, NULL)
+    INIT_TYPE(RefsIteratorType, NULL, NULL)
     ADD_TYPE(m, Object)
     ADD_TYPE(m, Commit)
     ADD_TYPE(m, Signature)
@@ -378,17 +550,27 @@ PyInit__pygit2(void)
     ADD_TYPE(m, TreeBuilder)
     ADD_TYPE(m, Blob)
     ADD_TYPE(m, Tag)
-    ADD_CONSTANT_INT(m, GIT_OBJ_ANY)
-    ADD_CONSTANT_INT(m, GIT_OBJ_COMMIT)
-    ADD_CONSTANT_INT(m, GIT_OBJ_TREE)
-    ADD_CONSTANT_INT(m, GIT_OBJ_BLOB)
-    ADD_CONSTANT_INT(m, GIT_OBJ_TAG)
+    ADD_CONSTANT_INT(m, GIT_OBJECT_ANY)
+    ADD_CONSTANT_INT(m, GIT_OBJECT_INVALID)
+    ADD_CONSTANT_INT(m, GIT_OBJECT_COMMIT)
+    ADD_CONSTANT_INT(m, GIT_OBJECT_TREE)
+    ADD_CONSTANT_INT(m, GIT_OBJECT_BLOB)
+    ADD_CONSTANT_INT(m, GIT_OBJECT_TAG)
+    ADD_CONSTANT_INT(m, GIT_OBJECT_OFS_DELTA)
+    ADD_CONSTANT_INT(m, GIT_OBJECT_REF_DELTA)
     /* Valid modes for index and tree entries. */
+    ADD_CONSTANT_INT(m, GIT_FILEMODE_UNREADABLE)
     ADD_CONSTANT_INT(m, GIT_FILEMODE_TREE)
     ADD_CONSTANT_INT(m, GIT_FILEMODE_BLOB)
     ADD_CONSTANT_INT(m, GIT_FILEMODE_BLOB_EXECUTABLE)
     ADD_CONSTANT_INT(m, GIT_FILEMODE_LINK)
     ADD_CONSTANT_INT(m, GIT_FILEMODE_COMMIT)
+    /* Deprecated constants for backward compatibility with old Python code*/
+    ADD_CONSTANT_INT(m, GIT_OBJ_ANY)
+    ADD_CONSTANT_INT(m, GIT_OBJ_COMMIT)
+    ADD_CONSTANT_INT(m, GIT_OBJ_TREE)
+    ADD_CONSTANT_INT(m, GIT_OBJ_BLOB)
+    ADD_CONSTANT_INT(m, GIT_OBJ_TAG)
 
     /*
      * Log
@@ -427,6 +609,10 @@ PyInit__pygit2(void)
     ADD_TYPE(m, Reference)
     ADD_TYPE(m, RefLogEntry)
     ADD_TYPE(m, Note)
+    ADD_CONSTANT_INT(m, GIT_REFERENCES_ALL)
+    ADD_CONSTANT_INT(m, GIT_REFERENCES_BRANCHES)
+    ADD_CONSTANT_INT(m, GIT_REFERENCES_TAGS)
+    /* libgit2 deprecated enums */
     ADD_CONSTANT_INT(m, GIT_REF_INVALID)
     ADD_CONSTANT_INT(m, GIT_REF_OID)
     ADD_CONSTANT_INT(m, GIT_REF_SYMBOLIC)
@@ -437,6 +623,10 @@ PyInit__pygit2(void)
      */
     INIT_TYPE(RevSpecType, NULL, NULL)
     ADD_TYPE(m, RevSpec)
+    ADD_CONSTANT_INT(m, GIT_REVSPEC_SINGLE)
+    ADD_CONSTANT_INT(m, GIT_REVSPEC_RANGE)
+    ADD_CONSTANT_INT(m, GIT_REVSPEC_MERGE_BASE)
+    /* libgit2 deprecated enums */
     ADD_CONSTANT_INT(m, GIT_REVPARSE_SINGLE)
     ADD_CONSTANT_INT(m, GIT_REVPARSE_RANGE)
     ADD_CONSTANT_INT(m, GIT_REVPARSE_MERGE_BASE)
@@ -477,16 +667,26 @@ PyInit__pygit2(void)
     /* Different checkout strategies */
     ADD_CONSTANT_INT(m, GIT_CHECKOUT_NONE)
     ADD_CONSTANT_INT(m, GIT_CHECKOUT_SAFE)
-    ADD_CONSTANT_INT(m, GIT_CHECKOUT_RECREATE_MISSING)
     ADD_CONSTANT_INT(m, GIT_CHECKOUT_FORCE)
+    ADD_CONSTANT_INT(m, GIT_CHECKOUT_RECREATE_MISSING)
     ADD_CONSTANT_INT(m, GIT_CHECKOUT_ALLOW_CONFLICTS)
     ADD_CONSTANT_INT(m, GIT_CHECKOUT_REMOVE_UNTRACKED)
     ADD_CONSTANT_INT(m, GIT_CHECKOUT_REMOVE_IGNORED)
     ADD_CONSTANT_INT(m, GIT_CHECKOUT_UPDATE_ONLY)
     ADD_CONSTANT_INT(m, GIT_CHECKOUT_DONT_UPDATE_INDEX)
     ADD_CONSTANT_INT(m, GIT_CHECKOUT_NO_REFRESH)
+    ADD_CONSTANT_INT(m, GIT_CHECKOUT_SKIP_UNMERGED)
+    ADD_CONSTANT_INT(m, GIT_CHECKOUT_USE_OURS)
+    ADD_CONSTANT_INT(m, GIT_CHECKOUT_USE_THEIRS)
     ADD_CONSTANT_INT(m, GIT_CHECKOUT_DISABLE_PATHSPEC_MATCH)
     ADD_CONSTANT_INT(m, GIT_CHECKOUT_SKIP_LOCKED_DIRECTORIES)
+    ADD_CONSTANT_INT(m, GIT_CHECKOUT_DONT_OVERWRITE_IGNORED)
+    ADD_CONSTANT_INT(m, GIT_CHECKOUT_CONFLICT_STYLE_MERGE)
+    ADD_CONSTANT_INT(m, GIT_CHECKOUT_CONFLICT_STYLE_DIFF3)
+    ADD_CONSTANT_INT(m, GIT_CHECKOUT_DONT_REMOVE_EXISTING)
+    ADD_CONSTANT_INT(m, GIT_CHECKOUT_DONT_WRITE_INDEX)
+    ADD_CONSTANT_INT(m, GIT_CHECKOUT_DRY_RUN)
+    ADD_CONSTANT_INT(m, GIT_CHECKOUT_CONFLICT_STYLE_ZDIFF3)
 
     /*
      * Diff
@@ -529,6 +729,7 @@ PyInit__pygit2(void)
     ADD_CONSTANT_INT(m, GIT_DIFF_INCLUDE_UNREADABLE)
     ADD_CONSTANT_INT(m, GIT_DIFF_INCLUDE_UNREADABLE_AS_UNTRACKED)
     ADD_CONSTANT_INT(m, GIT_DIFF_INDENT_HEURISTIC)
+    ADD_CONSTANT_INT(m, GIT_DIFF_IGNORE_BLANK_LINES)
     ADD_CONSTANT_INT(m, GIT_DIFF_FORCE_TEXT)
     ADD_CONSTANT_INT(m, GIT_DIFF_FORCE_BINARY)
     ADD_CONSTANT_INT(m, GIT_DIFF_IGNORE_WHITESPACE)
@@ -570,6 +771,7 @@ PyInit__pygit2(void)
     ADD_CONSTANT_INT(m, GIT_DIFF_FLAG_NOT_BINARY)
     ADD_CONSTANT_INT(m, GIT_DIFF_FLAG_VALID_ID)
     ADD_CONSTANT_INT(m, GIT_DIFF_FLAG_EXISTS)
+    ADD_CONSTANT_INT(m, GIT_DIFF_FLAG_VALID_SIZE)
 
     /* DiffDelta.status (git_delta_t in libgit2) */
     ADD_CONSTANT_INT(m, GIT_DELTA_UNMODIFIED)
@@ -585,10 +787,13 @@ PyInit__pygit2(void)
     ADD_CONSTANT_INT(m, GIT_DELTA_CONFLICTED)
 
     /* Config */
-    ADD_CONSTANT_INT(m, GIT_CONFIG_LEVEL_LOCAL);
-    ADD_CONSTANT_INT(m, GIT_CONFIG_LEVEL_GLOBAL);
-    ADD_CONSTANT_INT(m, GIT_CONFIG_LEVEL_XDG);
+    ADD_CONSTANT_INT(m, GIT_CONFIG_LEVEL_PROGRAMDATA);
     ADD_CONSTANT_INT(m, GIT_CONFIG_LEVEL_SYSTEM);
+    ADD_CONSTANT_INT(m, GIT_CONFIG_LEVEL_XDG);
+    ADD_CONSTANT_INT(m, GIT_CONFIG_LEVEL_GLOBAL);
+    ADD_CONSTANT_INT(m, GIT_CONFIG_LEVEL_LOCAL);
+    ADD_CONSTANT_INT(m, GIT_CONFIG_LEVEL_APP);
+    ADD_CONSTANT_INT(m, GIT_CONFIG_HIGHEST_LEVEL);
 
     /* Blame */
     ADD_CONSTANT_INT(m, GIT_BLAME_NORMAL)
@@ -620,6 +825,7 @@ PyInit__pygit2(void)
     ADD_CONSTANT_INT(m, GIT_STASH_KEEP_INDEX);
     ADD_CONSTANT_INT(m, GIT_STASH_INCLUDE_UNTRACKED);
     ADD_CONSTANT_INT(m, GIT_STASH_INCLUDE_IGNORED);
+    ADD_CONSTANT_INT(m, GIT_STASH_KEEP_ALL);
     ADD_CONSTANT_INT(m, GIT_STASH_APPLY_DEFAULT);
     ADD_CONSTANT_INT(m, GIT_STASH_APPLY_REINSTATE_INDEX);
 
@@ -628,6 +834,28 @@ PyInit__pygit2(void)
     ADD_CONSTANT_INT(m, GIT_APPLY_LOCATION_INDEX);
     ADD_CONSTANT_INT(m, GIT_APPLY_LOCATION_BOTH);
 
+    /* Submodule */
+    ADD_CONSTANT_INT(m, GIT_SUBMODULE_IGNORE_UNSPECIFIED);
+    ADD_CONSTANT_INT(m, GIT_SUBMODULE_IGNORE_NONE);
+    ADD_CONSTANT_INT(m, GIT_SUBMODULE_IGNORE_UNTRACKED);
+    ADD_CONSTANT_INT(m, GIT_SUBMODULE_IGNORE_DIRTY);
+    ADD_CONSTANT_INT(m, GIT_SUBMODULE_IGNORE_ALL);
+
+    ADD_CONSTANT_INT(m, GIT_SUBMODULE_STATUS_IN_HEAD);
+    ADD_CONSTANT_INT(m, GIT_SUBMODULE_STATUS_IN_INDEX);
+    ADD_CONSTANT_INT(m, GIT_SUBMODULE_STATUS_IN_CONFIG);
+    ADD_CONSTANT_INT(m, GIT_SUBMODULE_STATUS_IN_WD);
+    ADD_CONSTANT_INT(m, GIT_SUBMODULE_STATUS_INDEX_ADDED);
+    ADD_CONSTANT_INT(m, GIT_SUBMODULE_STATUS_INDEX_DELETED);
+    ADD_CONSTANT_INT(m, GIT_SUBMODULE_STATUS_INDEX_MODIFIED);
+    ADD_CONSTANT_INT(m, GIT_SUBMODULE_STATUS_WD_UNINITIALIZED);
+    ADD_CONSTANT_INT(m, GIT_SUBMODULE_STATUS_WD_ADDED);
+    ADD_CONSTANT_INT(m, GIT_SUBMODULE_STATUS_WD_DELETED);
+    ADD_CONSTANT_INT(m, GIT_SUBMODULE_STATUS_WD_MODIFIED);
+    ADD_CONSTANT_INT(m, GIT_SUBMODULE_STATUS_WD_INDEX_MODIFIED);
+    ADD_CONSTANT_INT(m, GIT_SUBMODULE_STATUS_WD_WD_MODIFIED);
+    ADD_CONSTANT_INT(m, GIT_SUBMODULE_STATUS_WD_UNTRACKED);
+
     /* Mailmap */
     INIT_TYPE(MailmapType, NULL, PyType_GenericNew)
     ADD_TYPE(m, Mailmap)
@@ -636,6 +864,25 @@ PyInit__pygit2(void)
     INIT_TYPE(StashType, NULL, NULL)
     ADD_TYPE(m, Stash)
 
+    /* Blob filter */
+    ADD_CONSTANT_INT(m, GIT_BLOB_FILTER_CHECK_FOR_BINARY);
+    ADD_CONSTANT_INT(m, GIT_BLOB_FILTER_NO_SYSTEM_ATTRIBUTES);
+    ADD_CONSTANT_INT(m, GIT_BLOB_FILTER_ATTRIBUTES_FROM_HEAD);
+    ADD_CONSTANT_INT(m, GIT_BLOB_FILTER_ATTRIBUTES_FROM_COMMIT);
+    ADD_CONSTANT_INT(m, GIT_FILTER_DRIVER_PRIORITY);
+    ADD_CONSTANT_INT(m, GIT_FILTER_TO_WORKTREE);
+    ADD_CONSTANT_INT(m, GIT_FILTER_SMUDGE);
+    ADD_CONSTANT_INT(m, GIT_FILTER_TO_ODB);
+    ADD_CONSTANT_INT(m, GIT_FILTER_CLEAN);
+    ADD_CONSTANT_INT(m, GIT_FILTER_DEFAULT);
+    ADD_CONSTANT_INT(m, GIT_FILTER_ALLOW_UNSAFE);
+    ADD_CONSTANT_INT(m, GIT_FILTER_NO_SYSTEM_ATTRIBUTES);
+    ADD_CONSTANT_INT(m, GIT_FILTER_ATTRIBUTES_FROM_HEAD);
+    ADD_CONSTANT_INT(m, GIT_FILTER_ATTRIBUTES_FROM_COMMIT);
+
+    INIT_TYPE(FilterSourceType, NULL, NULL);
+    ADD_TYPE(m, FilterSource);
+
     /* Global initialization of libgit2 */
     git_libgit2_init();
 
