@@ -1,5 +1,5 @@
 /*
- * Copyright 2010-2021 The pygit2 contributors
+ * Copyright 2010-2024 The pygit2 contributors
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2,
@@ -43,6 +43,7 @@ extern PyObject *GitError;
 extern PyTypeObject RefLogEntryType;
 extern PyTypeObject RepositoryType;
 extern PyTypeObject SignatureType;
+extern PyObject *ReferenceTypeEnum;
 
 PyTypeObject ReferenceType;
 
@@ -216,21 +217,22 @@ Reference_rename(Reference *self, PyObject *py_name)
 {
     CHECK_REFERENCE(self);
 
-    /* Get the C name */
+    // Get the C name
     char *c_name = pgit_encode_fsdefault(py_name);
     if (c_name == NULL)
         return NULL;
 
-    /* Rename */
+    // Rename
     git_reference *new_reference;
     int err = git_reference_rename(&new_reference, self->reference, c_name, 0, NULL);
     free(c_name);
-
-    git_reference_free(self->reference);
     if (err)
         return Error_set(err);
 
+    // Upadate reference
+    git_reference_free(self->reference);
     self->reference = new_reference;
+
     Py_RETURN_NONE;
 }
 
@@ -322,11 +324,7 @@ Reference_raw_target__get__(Reference *self)
 PyDoc_STRVAR(Reference_set_target__doc__,
     "set_target(target[, message: str])\n"
     "\n"
-    "Set the target of this reference.\n"
-    "\n"
-    "Update the reference using the given signature and message.\n"
-    "These will be used to fill the reflog entry which will be created\n"
-    "as a result of this update.\n"
+    "Set the target of this reference. Creates a new entry in the reflog.\n"
     "\n"
     "Parameters:\n"
     "\n"
@@ -427,7 +425,7 @@ Reference_raw_shorthand__get__(Reference *self)
 }
 
 PyDoc_STRVAR(Reference_type__doc__,
-    "Type, either GIT_REF_OID or GIT_REF_SYMBOLIC.");
+    "An enums.ReferenceType constant (either OID or SYMBOLIC).");
 
 PyObject *
 Reference_type__get__(Reference *self)
@@ -436,7 +434,8 @@ Reference_type__get__(Reference *self)
 
     CHECK_REFERENCE(self);
     c_type = git_reference_type(self->reference);
-    return PyLong_FromLong(c_type);
+
+    return pygit2_enum(ReferenceTypeEnum, c_type);
 }
 
 
@@ -488,7 +487,7 @@ Reference_peel(Reference *self, PyObject *args)
         return NULL;
 
     otype = py_object_to_otype(py_type);
-    if (otype == GIT_OBJ_BAD)
+    if (otype == GIT_OBJECT_INVALID)
         return NULL;
 
     err = git_reference_peel(&obj, self->reference, otype);
