@@ -1,5 +1,5 @@
 /*
- * Copyright 2010-2021 The pygit2 contributors
+ * Copyright 2010-2024 The pygit2 contributors
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2,
@@ -63,6 +63,11 @@ extern PyTypeObject RevSpecType;
 extern PyTypeObject NoteType;
 extern PyTypeObject NoteIterType;
 extern PyTypeObject StashType;
+extern PyTypeObject RefsIteratorType;
+
+extern PyObject *FileStatusEnum;
+extern PyObject *MergeAnalysisEnum;
+extern PyObject *MergePreferenceEnum;
 
 /* forward-declaration for Repsository._from_c() */
 PyTypeObject RepositoryType;
@@ -296,7 +301,7 @@ Repository_git_object_lookup_prefix(Repository *self, PyObject *key)
     if (len == 0)
         return NULL;
 
-    err = git_object_lookup_prefix(&obj, self->repo, &oid, len, GIT_OBJ_ANY);
+    err = git_object_lookup_prefix(&obj, self->repo, &oid, len, GIT_OBJECT_ANY);
     if (err == 0)
         return wrap_object(obj, self, NULL);
 
@@ -308,10 +313,10 @@ Repository_git_object_lookup_prefix(Repository *self, PyObject *key)
 
 
 PyDoc_STRVAR(Repository_lookup_branch__doc__,
-  "lookup_branch(branch_name: str, branch_type: int = GIT_BRANCH_LOCAL) -> Branch\n"
+  "lookup_branch(branch_name: str, branch_type: BranchType = BranchType.LOCAL) -> Branch\n"
   "\n"
   "Returns the Git reference for the given branch name (local or remote).\n"
-  "If branch_type is GIT_BRANCH_REMOTE, you must include the remote name\n"
+  "If branch_type is BranchType.REMOTE, you must include the remote name\n"
   "in the branch name (eg 'origin/master').");
 
 PyObject *
@@ -646,7 +651,7 @@ Repository_merge_base_octopus(Repository *self, PyObject *args)
 }
 
 PyDoc_STRVAR(Repository_merge_analysis__doc__,
-  "merge_analysis(their_head: Oid, our_ref: str = \"HEAD\") -> tuple[int, int]\n"
+  "merge_analysis(their_head: Oid, our_ref: str = \"HEAD\") -> tuple[MergeAnalysis, MergePreference]\n"
   "\n"
   "Analyzes the given branch and determines the opportunities for\n"
   "merging it into a reference (defaults to HEAD).\n"
@@ -659,8 +664,8 @@ PyDoc_STRVAR(Repository_merge_analysis__doc__,
   "their_head\n"
   "    Head (commit Oid) to merge into\n"
   "\n"
-  "The first returned value is a mixture of the GIT_MERGE_ANALYSIS_NONE, _NORMAL,\n"
-  "_UP_TO_DATE, _FASTFORWARD and _UNBORN flags.\n"
+  "The first returned value is a mixture of the MergeAnalysis.NONE, NORMAL,\n"
+  "UP_TO_DATE, FASTFORWARD and UNBORN flags.\n"
   "The second value is the user's preference from 'merge.ff'");
 
 PyObject *
@@ -704,54 +709,28 @@ Repository_merge_analysis(Repository *self, PyObject *args)
         goto out;
     }
 
-    py_result = Py_BuildValue("(ii)", analysis, preference);
+    // Convert analysis to MergeAnalysis enum
+    PyObject *analysis_enum = pygit2_enum(MergeAnalysisEnum, analysis);
+    if (!analysis_enum) {
+        py_result = NULL;
+        goto out;
+    }
+
+    // Convert preference to MergePreference enum
+    PyObject *preference_enum = pygit2_enum(MergePreferenceEnum, preference);
+    if (!preference_enum) {
+        py_result = NULL;
+        Py_DECREF(analysis_enum);
+        goto out;
+    }
+
+    py_result = Py_BuildValue("(OO)", analysis_enum, preference_enum);
 
 out:
     git_reference_free(our_ref);
     return py_result;
 }
 
-PyDoc_STRVAR(Repository_merge__doc__,
-  "merge(id: Oid)\n"
-  "\n"
-  "Merges the given id into HEAD.\n"
-  "\n"
-  "Merges the given commit(s) into HEAD, writing the results into the\n"
-  "working directory. Any changes are staged for commit and any conflicts\n"
-  "are written to the index. Callers should inspect the repository's\n"
-  "index after this completes, resolve any conflicts and prepare a\n"
-  "commit.");
-
-PyObject *
-Repository_merge(Repository *self, PyObject *py_oid)
-{
-    git_annotated_commit *commit;
-    git_oid oid;
-    int err;
-    size_t len;
-    git_merge_options merge_opts = GIT_MERGE_OPTIONS_INIT;
-    git_checkout_options checkout_opts = GIT_CHECKOUT_OPTIONS_INIT;
-
-    len = py_oid_to_git_oid(py_oid, &oid);
-    if (len == 0)
-        return NULL;
-
-    err = git_annotated_commit_lookup(&commit, self->repo, &oid);
-    if (err < 0)
-        return Error_set(err);
-
-    checkout_opts.checkout_strategy = GIT_CHECKOUT_SAFE | GIT_CHECKOUT_RECREATE_MISSING;
-    err = git_merge(self->repo,
-                    (const git_annotated_commit **)&commit, 1,
-                    &merge_opts, &checkout_opts);
-
-    git_annotated_commit_free(commit);
-    if (err < 0)
-        return Error_set(err);
-
-    Py_RETURN_NONE;
-}
-
 PyDoc_STRVAR(Repository_cherrypick__doc__,
   "cherrypick(id: Oid)\n"
   "\n"
@@ -793,31 +772,30 @@ Repository_cherrypick(Repository *self, PyObject *py_oid)
 }
 
 PyDoc_STRVAR(Repository_walk__doc__,
-    "walk(oid: Oid | None, sort_mode: int = GIT_SORT_NONE) -> Walker\n"
+    "walk(oid: Oid | None, sort_mode: enums.SortMode = enums.SortMode.NONE) -> Walker\n"
     "\n"
     "Start traversing the history from the given commit.\n"
-    "The following types of sorting could be used to control traversing\n"
-    "direction:\n"
+    "The following SortMode values can be used to control the walk:\n"
     "\n"
-    "* GIT_SORT_NONE. Sort the output with the same default method from\n"
+    "* NONE. Sort the output with the same default method from\n"
     "  `git`: reverse chronological order. This is the default sorting for\n"
     "  new walkers.\n"
-    "* GIT_SORT_TOPOLOGICAL. Sort the repository contents in topological order\n"
+    "* TOPOLOGICAL. Sort the repository contents in topological order\n"
     "  (no parents before all of its children are shown); this sorting mode\n"
     "  can be combined with time sorting to produce `git`'s `--date-order``.\n"
-    "* GIT_SORT_TIME. Sort the repository contents by commit time; this sorting\n"
+    "* TIME. Sort the repository contents by commit time; this sorting\n"
     "  mode can be combined with topological sorting.\n"
-    "* GIT_SORT_REVERSE.  Iterate through the repository contents in reverse\n"
+    "* REVERSE.  Iterate through the repository contents in reverse\n"
     "  order; this sorting mode can be combined with any of the above.\n"
     "\n"
     "Example:\n"
     "\n"
     "  >>> from pygit2 import Repository\n"
-    "  >>> from pygit2 import GIT_SORT_TOPOLOGICAL, GIT_SORT_REVERSE\n"
+    "  >>> from pygit2.enums import SortMode\n"
     "  >>> repo = Repository('.git')\n"
-    "  >>> for commit in repo.walk(repo.head.target, GIT_SORT_TOPOLOGICAL):\n"
+    "  >>> for commit in repo.walk(repo.head.target, SortMode.TOPOLOGICAL):\n"
     "  ...    print(commit.message)\n"
-    "  >>> for commit in repo.walk(repo.head.target, GIT_SORT_TOPOLOGICAL | GIT_SORT_REVERSE):\n"
+    "  >>> for commit in repo.walk(repo.head.target, SortMode.TOPOLOGICAL | SortMode.REVERSE):\n"
     "  ...    print(commit.message)\n"
     "  >>>\n");
 
@@ -1121,7 +1099,7 @@ Repository_create_commit_string(Repository *self, PyObject *args)
 {
     Signature *py_author, *py_committer;
     PyObject *py_oid, *py_message, *py_parents, *py_parent;
-    PyObject *str;
+    PyObject *str = NULL;
     char *encoding = NULL;
     git_oid oid;
     git_tree *tree = NULL;
@@ -1196,7 +1174,7 @@ out:
 }
 
 PyDoc_STRVAR(Repository_create_commit_with_signature__doc__,
-  "create_commit_with_signature(content: str, signature: str, field_name: str) -> Oid\n"
+  "create_commit_with_signature(content: str, signature: str[, field_name: str]) -> Oid\n"
   "\n"
   "Create a new signed commit object, return its oid.");
 
@@ -1222,7 +1200,7 @@ Repository_create_commit_with_signature(Repository *self, PyObject *args)
 }
 
 PyDoc_STRVAR(Repository_create_tag__doc__,
-  "create_tag(name: str, oid: Oid, type: int, tagger: Signature[, message: str]) -> Oid\n"
+  "create_tag(name: str, oid: Oid, type: enums.ObjectType, tagger: Signature[, message: str]) -> Oid\n"
   "\n"
   "Create a new tag object, return its oid.");
 
@@ -1301,8 +1279,13 @@ to_path_f(const char * x) {
     return to_path(x);
 }
 
+PyDoc_STRVAR(Repository_raw_listall_references__doc__,
+  "raw_listall_references() -> list[bytes]\n"
+  "\n"
+  "Return a list with all the references in the repository.");
+
 static PyObject *
-Repository_listall_references_impl(Repository *self, PyObject *args, PyObject *(*item_trans)(const char *))
+Repository_raw_listall_references(Repository *self, PyObject *args)
 {
     git_strarray c_result;
     PyObject *py_result, *py_string;
@@ -1321,7 +1304,7 @@ Repository_listall_references_impl(Repository *self, PyObject *args, PyObject *(
 
     /* Fill it */
     for (index=0; index < c_result.count; index++) {
-        py_string = item_trans(c_result.strings[index]);
+        py_string = PyBytes_FromString(c_result.strings[index]);
         if (py_string == NULL) {
             Py_CLEAR(py_result);
             goto out;
@@ -1334,78 +1317,92 @@ out:
     return py_result;
 }
 
-PyDoc_STRVAR(Repository_listall_references__doc__,
-  "listall_references() -> list[str]\n"
-  "\n"
-  "Return a list with all the references in the repository.");
 
 PyObject *
-Repository_listall_references(Repository *self, PyObject *args)
-{
-    return Repository_listall_references_impl(self, args, to_path_f);
+wrap_references_iterator(git_reference_iterator *iter) {
+    RefsIterator *py_refs_iter = PyObject_New(RefsIterator , &ReferenceType);
+    if (py_refs_iter)
+        py_refs_iter->iterator = iter;
+
+    return (PyObject *)py_refs_iter;
 }
 
-PyDoc_STRVAR(Repository_raw_listall_references__doc__,
-  "raw_listall_references() -> list[bytes]\n"
-  "\n"
-  "Return a list with all the references in the repository.");
-
-PyObject *
-Repository_raw_listall_references(Repository *self, PyObject *args)
+void
+References_iterator_dealloc(RefsIterator *iter)
 {
-    return Repository_listall_references_impl(self, args, PyBytes_FromString);
+    git_reference_iterator_free(iter->iterator);
+    Py_TYPE(iter)->tp_free((PyObject *)iter);
 }
 
-
-PyDoc_STRVAR(Repository_listall_reference_objects__doc__,
-  "listall_reference_objects() -> list[Reference]\n"
+PyDoc_STRVAR(Repository_references_iterator_init__doc__,
+  "references_iterator_init() -> git_reference_iterator\n"
   "\n"
-  "Return a list with all the reference objects in the repository.");
+  "Creates and returns an iterator for references.");
 
 PyObject *
-Repository_listall_reference_objects(Repository *self, PyObject *args)
+Repository_references_iterator_init(Repository *self, PyObject *args)
 {
-    git_reference_iterator *iter;
-    git_reference *ref = NULL;
     int err;
-    PyObject *list;
+    git_reference_iterator *iter;
+    RefsIterator *refs_iter;
 
-    list = PyList_New(0);
-    if (list == NULL)
+    refs_iter = PyObject_New(RefsIterator, &RefsIteratorType);
+    if (refs_iter == NULL) {
         return NULL;
+    }
 
     if ((err = git_reference_iterator_new(&iter, self->repo)) < 0)
         return Error_set(err);
 
-    while ((err = git_reference_next(&ref, iter)) == 0) {
-        PyObject *py_ref = wrap_reference(ref, self);
-        if (py_ref == NULL)
-            goto error;
-
-        err = PyList_Append(list, py_ref);
-        Py_DECREF(py_ref);
-
-        if (err < 0)
-            goto error;
-    }
-
-    git_reference_iterator_free(iter);
-    if (err == GIT_ITEROVER)
-        err = 0;
-
-    if (err < 0) {
-        Py_CLEAR(list);
-        return Error_set(err);
-    }
-
-    return list;
-
-error:
-    git_reference_iterator_free(iter);
-    Py_CLEAR(list);
-    return NULL;
+    refs_iter->iterator = iter;
+    return (PyObject*)refs_iter;
 }
 
+PyDoc_STRVAR(Repository_references_iterator_next__doc__,
+  "references_iterator_next(iter: Iterator[Reference], references_return_type: ReferenceFilter = ReferenceFilter.ALL) -> Reference\n"
+  "\n"
+  "Returns next reference object for repository. Optionally, can filter \n"
+  "based on value of references_return_type.\n"
+  "Acceptable values of references_return_type:\n"
+  "ReferenceFilter.ALL -> returns all refs, this is the default\n"
+  "ReferenceFilter.BRANCHES -> returns all branches\n"
+  "ReferenceFilter.TAGS -> returns all tags\n"
+  "all other values -> will return None");
+
+PyObject *
+Repository_references_iterator_next(Repository *self, PyObject *args)
+{
+    git_reference *ref;
+    git_reference_iterator *git_iter;
+    PyObject *iter;
+    int references_return_type = GIT_REFERENCES_ALL;
+
+    if (!PyArg_ParseTuple(args, "O|i", &iter, &references_return_type))
+        return NULL;
+    git_iter = ((RefsIterator *) iter)->iterator;
+
+    int err;
+    while (0 == (err = git_reference_next(&ref, git_iter))) {
+        switch(references_return_type) {
+            case GIT_REFERENCES_ALL:
+                return wrap_reference(ref, self);
+            case GIT_REFERENCES_BRANCHES:
+                if (git_reference_is_branch(ref)) {
+                    return wrap_reference(ref, self);
+                }
+                break;
+            case GIT_REFERENCES_TAGS:
+                if (git_reference_is_tag(ref)) {
+                    return wrap_reference(ref, self);
+                }
+                break;
+        }
+    }
+    if (err == GIT_ITEROVER) {
+        Py_RETURN_NONE;
+    }
+    return Error_set(err);
+}
 
 static PyObject *
 Repository_listall_branches_impl(Repository *self, PyObject *args, PyObject *(*item_trans)(const char *))
@@ -1460,15 +1457,15 @@ error:
 }
 
 PyDoc_STRVAR(Repository_listall_branches__doc__,
-  "listall_branches(flag: int = GIT_BRANCH_LOCAL) -> list[str]\n"
+  "listall_branches(flag: BranchType = BranchType.LOCAL) -> list[str]\n"
   "\n"
   "Return a list with all the branches in the repository.\n"
   "\n"
   "The *flag* may be:\n"
   "\n"
-  "- GIT_BRANCH_LOCAL - return all local branches (set by default)\n"
-  "- GIT_BRANCH_REMOTE - return all remote-tracking branches\n"
-  "- GIT_BRANCH_ALL - return local branches and remote-tracking branches");
+  "- BranchType.LOCAL - return all local branches (set by default)\n"
+  "- BranchType.REMOTE - return all remote-tracking branches\n"
+  "- BranchType.ALL - return local branches and remote-tracking branches");
 
 PyObject *
 Repository_listall_branches(Repository *self, PyObject *args)
@@ -1477,15 +1474,15 @@ Repository_listall_branches(Repository *self, PyObject *args)
 }
 
 PyDoc_STRVAR(Repository_raw_listall_branches__doc__,
-  "raw_listall_branches(flag: int = GIT_BRANCH_LOCAL) -> list[bytes]\n"
+  "raw_listall_branches(flag: BranchType = BranchType.LOCAL) -> list[bytes]\n"
   "\n"
   "Return a list with all the branches in the repository.\n"
   "\n"
   "The *flag* may be:\n"
   "\n"
-  "- GIT_BRANCH_LOCAL - return all local branches (set by default)\n"
-  "- GIT_BRANCH_REMOTE - return all remote-tracking branches\n"
-  "- GIT_BRANCH_ALL - return local branches and remote-tracking branches");
+  "- BranchType.LOCAL - return all local branches (set by default)\n"
+  "- BranchType.REMOTE - return all remote-tracking branches\n"
+  "- BranchType.ALL - return local branches and remote-tracking branches");
 
 PyObject *
 Repository_raw_listall_branches(Repository *self, PyObject *args)
@@ -1528,81 +1525,6 @@ Repository_listall_submodules(Repository *self, PyObject *args)
 }
 
 
-PyDoc_STRVAR(Repository_init_submodules__doc__,
-    "init_submodules(submodules: list[Submodule] = None, overwrite=False)\n"
-    "\n"
-    "Initialize all submodules in repository.\n"
-    "submodules: List of submodules to initialize. Default argument initializes all submodules.\n"
-    "overwrite: Flag indicating if initialization should overwrite submodule entries.\n");
-
-static int foreach_sub_init_cb(git_submodule *submodule, const char *name, void *payload)
-{
-    return git_submodule_init(submodule, *(int*)payload);
-}
-
-PyObject *
-Repository_init_submodules(Repository* self, PyObject *args, PyObject *kwds)
-{
-    PyObject *list = Py_None;
-    PyObject *oflag = Py_False;
-    char *kwlist[] = {"submodules", "overwrite", NULL};
-    int err;
-    const char *c_subpath;
-    git_submodule *submodule;
-
-    if (!PyArg_ParseTupleAndKeywords(args, kwds, "|OO", kwlist, &list, &oflag))
-        return NULL;
-
-    int fflag = PyObject_IsTrue(oflag);
-    if (fflag != 0 && fflag != 1)
-        fflag = 0;
-
-    //Init all submodules listed in repository
-    if (list == Py_None) {
-        err = git_submodule_foreach(self->repo, foreach_sub_init_cb, &fflag);
-        if (err != 0)
-            return Error_set(err);
-        Py_RETURN_NONE;
-    }
-
-    PyObject *iter = PyObject_GetIter(list);
-    if (!iter)
-        return NULL;
-
-    PyObject *next = NULL;
-    while (1) {
-        Py_XDECREF(next); // Decref from the previous iteration
-        next = PyIter_Next(iter);
-        if (next == NULL) // List exhausted
-            break;
-
-        c_subpath = pgit_borrow(next);
-        if (c_subpath == NULL)
-            goto error;
-
-        err = git_submodule_lookup(&submodule, self->repo, c_subpath);
-        if (!submodule) {
-            PyErr_SetString(PyExc_KeyError, "Submodule does not exist");
-            goto error;
-        }
-
-        err = git_submodule_init(submodule, fflag);
-        if (err) {
-            Error_set(err);
-            goto error;
-        }
-    }
-
-    // Success
-    Py_DECREF(iter);
-    Py_RETURN_NONE;
-
-error:
-    Py_DECREF(iter);
-    Py_XDECREF(next);
-    return NULL;
-}
-
 PyDoc_STRVAR(Repository_lookup_reference__doc__,
   "lookup_reference(name: str) -> Reference\n"
   "\n"
@@ -1772,10 +1694,10 @@ Repository_compress_references(Repository *self)
 }
 
 PyDoc_STRVAR(Repository_status__doc__,
-  "status(untracked_files: str = \"all\", ignored: bool = False) -> dict[str, int]\n"
+  "status(untracked_files: str = \"all\", ignored: bool = False) -> dict[str, enums.FileStatus]\n"
   "\n"
   "Reads the status of the repository and returns a dictionary with file\n"
-  "paths as keys and status flags as values. See pygit2.GIT_STATUS_*.\n"
+  "paths as keys and FileStatus flags as values.\n"
   "\n"
   "Parameters:\n"
   "\n"
@@ -1796,7 +1718,6 @@ PyDoc_STRVAR(Repository_status__doc__,
 PyObject *
 Repository_status(Repository *self, PyObject *args, PyObject *kw)
 {
-    PyObject *dict;
     int err;
     size_t len, i;
     git_status_list *list;
@@ -1806,9 +1727,8 @@ Repository_status(Repository *self, PyObject *args, PyObject *kw)
 
     PyObject* ignored = Py_False;
 
-    if (!PyArg_ParseTupleAndKeywords(args, kw, "|sO", kwlist,
-                                     &untracked_files, &ignored))
-        goto error;
+    if (!PyArg_ParseTupleAndKeywords(args, kw, "|sO", kwlist, &untracked_files, &ignored))
+        return NULL;
 
     git_status_options opts = GIT_STATUS_OPTIONS_INIT;
     opts.flags = GIT_STATUS_OPT_DEFAULTS;
@@ -1834,7 +1754,7 @@ Repository_status(Repository *self, PyObject *args, PyObject *kw)
     if (err < 0)
         return Error_set(err);
 
-    dict = PyDict_New();
+    PyObject *dict = PyDict_New();
     if (dict == NULL)
         goto error;
 
@@ -1853,14 +1773,17 @@ Repository_status(Repository *self, PyObject *args, PyObject *kw)
             path = entry->head_to_index->old_file.path;
         else
             path = entry->index_to_workdir->old_file.path;
-        status = PyLong_FromLong((long) entry->status);
+
+        /* Get corresponding entry in enums.FileStatus for status int */
+        status = pygit2_enum(FileStatusEnum, entry->status);
+        if (status == NULL)
+            goto error;
 
         err = PyDict_SetItemString(dict, path, status);
         Py_CLEAR(status);
 
         if (err < 0)
             goto error;
-
     }
 
     git_status_list_free(list);
@@ -1874,7 +1797,7 @@ error:
 
 
 PyDoc_STRVAR(Repository_status_file__doc__,
-  "status_file(path: str) -> int\n"
+  "status_file(path: str) -> enums.FileStatus\n"
   "\n"
   "Returns the status of the given file path.");
 
@@ -1894,7 +1817,7 @@ Repository_status_file(Repository *self, PyObject *value)
     }
     free(path);
 
-    return PyLong_FromLong(status);
+    return pygit2_enum(FileStatusEnum, (int) status);
 }
 
 
@@ -2094,7 +2017,7 @@ Repository_lookup_note(Repository *self, PyObject* args)
 }
 
 PyDoc_STRVAR(Repository_reset__doc__,
-    "reset(oid: Oid, reset_type: int)\n"
+    "reset(oid: Oid, reset_mode: enums.ResetMode)\n"
     "\n"
     "Resets the current head.\n"
     "\n"
@@ -2103,12 +2026,12 @@ PyDoc_STRVAR(Repository_reset__doc__,
     "oid\n"
     "    The oid of the commit to reset to.\n"
     "\n"
-    "reset_type\n"
-    "    * GIT_RESET_SOFT: resets head to point to oid, but does not modify\n"
+    "reset_mode\n"
+    "    * SOFT: resets head to point to oid, but does not modify\n"
     "      working copy, and leaves the changes in the index.\n"
-    "    * GIT_RESET_MIXED: resets head to point to oid, but does not modify\n"
+    "    * MIXED: resets head to point to oid, but does not modify\n"
     "      working copy. It empties the index too.\n"
-    "    * GIT_RESET_HARD: resets head to point to oid, and resets too the\n"
+    "    * HARD: resets head to point to oid, and resets too the\n"
     "      working copy and the content of the index.\n");
 
 PyObject *
@@ -2131,7 +2054,7 @@ Repository_reset(Repository *self, PyObject* args)
         return NULL;
 
     err = git_object_lookup_prefix(&target, self->repo, &oid, len,
-                                   GIT_OBJ_ANY);
+                                   GIT_OBJECT_ANY);
     err = err < 0 ? err : git_reset(self->repo, target, reset_type, NULL);
     git_object_free(target);
     if (err < 0)
@@ -2264,7 +2187,7 @@ out:
 }
 
 PyDoc_STRVAR(Repository_apply__doc__,
-  "apply(diff: Diff, location: int = GIT_APPLY_LOCATION_WORKDIR)\n"
+  "apply(diff: Diff, location: ApplyLocation = ApplyLocation.WORKDIR)\n"
   "\n"
   "Applies the given Diff object to HEAD, writing the results into the\n"
   "working directory, the index, or both.\n"
@@ -2275,8 +2198,8 @@ PyDoc_STRVAR(Repository_apply__doc__,
   "    The Diff to apply.\n"
   "\n"
   "location\n"
-  "    The location to apply: GIT_APPLY_LOCATION_WORKDIR (default),\n"
-  "    GIT_APPLY_LOCATION_INDEX, or GIT_APPLY_LOCATION_BOTH.\n"
+  "    The location to apply: ApplyLocation.WORKDIR (default),\n"
+  "    ApplyLocation.INDEX, or ApplyLocation.BOTH.\n"
   );
 
 PyObject *
@@ -2435,6 +2358,52 @@ Repository_listall_stashes(Repository *self, PyObject *args)
     }
 }
 
+static int foreach_mergehead_cb(const git_oid *oid, void *payload)
+{
+    PyObject* py_oid = git_oid_to_python(oid);
+    if (py_oid == NULL)
+        return GIT_EUSER;
+
+    PyObject* list = (PyObject*) payload;
+    int err = PyList_Append(list, (PyObject*) py_oid);
+    Py_DECREF(py_oid);
+    if (err < 0)
+        return GIT_EUSER;
+
+    return 0;
+}
+
+PyDoc_STRVAR(Repository_listall_mergeheads__doc__,
+  "listall_mergeheads() -> list[Oid]\n"
+  "\n"
+  "If a merge is in progress, return a list of all commit oids in the MERGE_HEAD file.\n"
+  "Return an empty list if there is no MERGE_HEAD file (no merge in progress).");
+
+PyObject *
+Repository_listall_mergeheads(Repository *self, PyObject *args)
+{
+    int err;
+
+    PyObject *list = PyList_New(0);
+    if (list == NULL)
+        return NULL;
+
+    err = git_repository_mergehead_foreach(self->repo, foreach_mergehead_cb, (void*)list);
+
+    if (err == 0) {
+        return list;
+    } else if (err == GIT_ENOTFOUND) {
+        /* MERGE_HEAD not found - return empty list */
+        return list;
+    }
+    else {
+        Py_CLEAR(list);
+        if (PyErr_Occurred())
+            return NULL;
+        return Error_set(err);
+    }
+}
+
 PyMethodDef Repository_methods[] = {
     METHOD(Repository, create_blob, METH_VARARGS),
     METHOD(Repository, create_blob_fromworkdir, METH_O),
@@ -2451,18 +2420,16 @@ PyMethodDef Repository_methods[] = {
     METHOD(Repository, merge_base_many, METH_VARARGS),
     METHOD(Repository, merge_base_octopus, METH_VARARGS),
     METHOD(Repository, merge_analysis, METH_VARARGS),
-    METHOD(Repository, merge, METH_O),
     METHOD(Repository, cherrypick, METH_O),
     METHOD(Repository, apply, METH_VARARGS | METH_KEYWORDS),
     METHOD(Repository, applies, METH_VARARGS | METH_KEYWORDS),
     METHOD(Repository, create_reference_direct, METH_VARARGS | METH_KEYWORDS),
     METHOD(Repository, create_reference_symbolic, METH_VARARGS | METH_KEYWORDS),
     METHOD(Repository, compress_references, METH_NOARGS),
-    METHOD(Repository, listall_references, METH_NOARGS),
     METHOD(Repository, raw_listall_references, METH_NOARGS),
-    METHOD(Repository, listall_reference_objects, METH_NOARGS),
+    METHOD(Repository, references_iterator_init, METH_NOARGS),
+    METHOD(Repository, references_iterator_next, METH_VARARGS),
     METHOD(Repository, listall_submodules, METH_NOARGS),
-    METHOD(Repository, init_submodules, METH_VARARGS | METH_KEYWORDS),
     METHOD(Repository, lookup_reference, METH_O),
     METHOD(Repository, lookup_reference_dwim, METH_O),
     METHOD(Repository, revparse_single, METH_O),
@@ -2490,6 +2457,7 @@ PyMethodDef Repository_methods[] = {
     METHOD(Repository, set_odb, METH_O),
     METHOD(Repository, set_refdb, METH_O),
     METHOD(Repository, listall_stashes, METH_NOARGS),
+    METHOD(Repository, listall_mergeheads, METH_NOARGS),
     {NULL}
 };
 
@@ -2557,3 +2525,35 @@ PyTypeObject RepositoryType = {
     0,                                         /* tp_alloc          */
     0,                                         /* tp_new            */
 };
+
+PyDoc_STRVAR(RefsIterator__doc__, "References iterator.");
+
+PyTypeObject RefsIteratorType = {
+    PyVarObject_HEAD_INIT(NULL, 0)
+    "_pygit2.RefsIterator",                    /* tp_name           */
+    sizeof(Repository),                        /* tp_basicsize      */
+    0,                                         /* tp_itemsize       */
+    (destructor)References_iterator_dealloc,   /* tp_dealloc        */
+    0,                                         /* tp_print          */
+    0,                                         /* tp_getattr        */
+    0,                                         /* tp_setattr        */
+    0,                                         /* tp_compare        */
+    0,                                         /* tp_repr           */
+    0,                                         /* tp_as_number      */
+    0,                                         /* tp_as_sequence    */
+    0,                                         /* tp_as_mapping     */
+    0,                                         /* tp_hash           */
+    0,                                         /* tp_call           */
+    0,                                         /* tp_str            */
+    0,                                         /* tp_getattro       */
+    0,                                         /* tp_setattro       */
+    0,                                         /* tp_as_buffer      */
+    Py_TPFLAGS_DEFAULT,                        /* tp_flags          */
+    RefsIterator__doc__,                       /* tp_doc            */
+    0,                                         /* tp_traverse       */
+    0,                                         /* tp_clear          */
+    0,                                         /* tp_richcompare    */
+    0,                                         /* tp_weaklistoffset */
+    PyObject_SelfIter,                         /* tp_iter           */
+    (iternextfunc)Repository_references_iterator_next, /* tp_iternext */
+};
