@@ -1,5 +1,5 @@
 /*
- * Copyright 2010-2021 The pygit2 contributors
+ * Copyright 2010-2024 The pygit2 contributors
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2,
@@ -31,9 +31,10 @@
 #define PY_SSIZE_T_CLEAN
 #include <Python.h>
 #include <git2.h>
+#include <git2/sys/filter.h>
 
-#if !(LIBGIT2_VER_MAJOR == 1 && LIBGIT2_VER_MINOR == 5)
-#error You need a compatible libgit2 version (1.5.x)
+#if !(LIBGIT2_VER_MAJOR == 1 && LIBGIT2_VER_MINOR == 7)
+#error You need a compatible libgit2 version (1.7.x)
 #endif
 
 /*
@@ -88,6 +89,10 @@ typedef struct {
     RefdbBackend super;
 } RefdbFsBackend;
 
+typedef struct {
+    PyObject_HEAD
+    git_reference_iterator *iterator;
+} RefsIterator;
 
 #define SIMPLE_TYPE(_name, _ptr_type, _ptr_name) \
         typedef struct {\
@@ -262,4 +267,9 @@ typedef struct {
     char *message;
 } Stash;
 
+typedef struct {
+    PyObject_HEAD
+    const git_filter_source *src;
+} FilterSource;
+
 #endif
