@@ -1,5 +1,5 @@
 /*
- * Copyright 2010-2019 The pygit2 contributors
+ * Copyright 2010-2024 The pygit2 contributors
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2,
@@ -37,59 +37,136 @@ extern PyTypeObject BlobType;
 extern PyTypeObject TagType;
 
 /**
- * py_str_to_c_str() returns a newly allocated C string holding the string
- * contained in the 'value' argument.
+ * Attempt to convert a C string to a Python string with the given encoding.
+ * If the conversion fails, return a fallback string.
+ */
+PyObject *
+to_unicode_safe(const char *value, const char *encoding)
+{
+    PyObject *py_str;
+
+    if (!value) {
+        py_str = PyUnicode_FromString("None");
+    } else {
+        py_str = to_unicode(value, encoding, "replace");
+
+        if (!py_str) {
+            assert(PyErr_Occurred());
+            py_str = PyUnicode_FromString("(error)");
+            PyErr_Clear();
+        }
+    }
+
+    assert(!PyErr_Occurred());
+    assert(py_str);
+
+    return py_str;
+}
+
+/**
+ * Return a *newly allocated* C string holding the string contained in the
+ * 'value' argument.
  */
 char *
-py_str_to_c_str(PyObject *value, const char *encoding)
+pgit_encode(PyObject *value, const char *encoding)
 {
-    const char *borrowed;
-    char *c_str = NULL;
     PyObject *tmp = NULL;
 
-    borrowed = py_str_borrow_c_str(&tmp, value, encoding);
+    const char *borrowed = pgit_borrow_encoding(value, encoding, NULL, &tmp);
     if (!borrowed)
         return NULL;
 
-    c_str = strdup(borrowed);
-
+    char *c_str = strdup(borrowed);
     Py_DECREF(tmp);
+
+    if (!c_str) {
+        // FIXME Set exception
+    }
+
     return c_str;
 }
 
+char*
+pgit_encode_fsdefault(PyObject *value)
+{
+    return pgit_encode(value, Py_FileSystemDefaultEncoding);
+}
+
 /**
  * Return a pointer to the underlying C string in 'value'. The pointer is
- * guaranteed by 'tvalue'. Decrease its refcount when done with the string.
+ * guaranteed by 'tvalue', decrease its refcount when done with the string.
  */
-const char *
-py_str_borrow_c_str(PyObject **tvalue, PyObject *value, const char *encoding)
+const char*
+pgit_borrow_encoding(PyObject *value, const char *encoding, const char *errors, PyObject **tvalue)
 {
-    /* Case 1: byte string */
-    if (PyBytes_Check(value)) {
-        Py_INCREF(value);
-        *tvalue = value;
+    PyObject *py_value = NULL;
+    PyObject *py_str = NULL;
+
+    py_value = PyOS_FSPath(value);
+    if (py_value == NULL) {
+        Error_type_error("unexpected %.200s", value);
+        return NULL;
+    }
+
+    // Get new PyBytes reference from value
+    if (PyUnicode_Check(py_value)) { // Text string
+        py_str = PyUnicode_AsEncodedString(
+            py_value,
+            encoding ? encoding : "utf-8",
+            errors ? errors : "strict"
+        );
+
+        Py_DECREF(py_value);
+        if (py_str == NULL)
+            return NULL;
+    } else if (PyBytes_Check(py_value)) { // Byte string
+        py_str = py_value;
+    } else { // Type error
+        Error_type_error("unexpected %.200s", value);
+        Py_DECREF(py_value);
+        return NULL;
+    }
+
+    // Borrow c string from the new PyBytes reference
+    char *c_str = PyBytes_AsString(py_str);
+    if (c_str == NULL) {
+        Py_DECREF(py_str);
+        return NULL;
+    }
+
+    // Return the borrowed c string and the new PyBytes reference
+    *tvalue = py_str;
+    return c_str;
+}
+
+
+/**
+ * Return a borrowed c string with the representation of the given Unicode or
+ * Bytes object:
+ * - If value is Unicode return the UTF-8 representation
+ * - If value is Bytes return the raw sttring
+ * In both cases the returned string is owned by value and must not be
+ * modified, nor freed.
+ */
+const char*
+pgit_borrow(PyObject *value)
+{
+    if (PyUnicode_Check(value)) { // Text string
+        return PyUnicode_AsUTF8(value);
+    } else if (PyBytes_Check(value)) { // Byte string
         return PyBytes_AsString(value);
     }
 
-    /* Case 2: text string */
-    if (PyUnicode_Check(value)) {
-        if (encoding == NULL)
-            *tvalue = PyUnicode_AsUTF8String(value);
-        else
-            *tvalue = PyUnicode_AsEncodedString(value, encoding, "strict");
-        if (*tvalue == NULL)
-            return NULL;
-        return PyBytes_AsString(*tvalue);
-    }
-
-    /* Type error */
+    // Type error
     Error_type_error("unexpected %.200s", value);
     return NULL;
 }
 
+
 /**
  * Converts the (struct) git_strarray to a Python list
  */
+/*
 PyObject *
 get_pylist_from_git_strarray(git_strarray *strarray)
 {
@@ -106,11 +183,13 @@ get_pylist_from_git_strarray(git_strarray *strarray)
 
     return new_list;
 }
+*/
 
 /**
  * Converts the Python list to struct git_strarray
  * returns -1 if conversion failed
  */
+/*
 int
 get_strarraygit_from_pylist(git_strarray *array, PyObject *pylist)
 {
@@ -126,7 +205,7 @@ get_strarraygit_from_pylist(git_strarray *array, PyObject *pylist)
 
     n = PyList_Size(pylist);
 
-    /* allocate new git_strarray */
+    // allocate new git_strarray
     ptr = calloc(n, sizeof(char *));
     if (!ptr) {
         PyErr_SetNone(PyExc_MemoryError);
@@ -138,7 +217,7 @@ get_strarraygit_from_pylist(git_strarray *array, PyObject *pylist)
 
     for (index = 0; index < n; index++) {
         item = PyList_GetItem(pylist, index);
-        str = py_str_to_c_str(item, NULL);
+        str = pgit_encode(item, NULL);
         if (!str)
             goto on_error;
 
@@ -156,21 +235,22 @@ on_error:
 
     return -1;
 }
+*/
 
 static git_otype
 py_type_to_git_type(PyTypeObject *py_type)
 {
     if (py_type == &CommitType)
-        return GIT_OBJ_COMMIT;
+        return GIT_OBJECT_COMMIT;
     else if (py_type == &TreeType)
-        return GIT_OBJ_TREE;
+        return GIT_OBJECT_TREE;
     else if (py_type == &BlobType)
-        return GIT_OBJ_BLOB;
+        return GIT_OBJECT_BLOB;
     else if (py_type == &TagType)
-        return GIT_OBJ_TAG;
+        return GIT_OBJECT_TAG;
 
     PyErr_SetString(PyExc_ValueError, "invalid target type");
-    return GIT_OBJ_BAD; /* -1 */
+    return GIT_OBJECT_INVALID; /* -1 */
 }
 
 git_otype
@@ -179,12 +259,12 @@ py_object_to_otype(PyObject *py_type)
     long value;
 
     if (py_type == Py_None)
-        return GIT_OBJ_ANY;
+        return GIT_OBJECT_ANY;
 
     if (PyLong_Check(py_type)) {
         value = PyLong_AsLong(py_type);
         if (value == -1 && PyErr_Occurred())
-            return GIT_OBJ_BAD;
+            return GIT_OBJECT_INVALID;
 
         /* TODO Check whether the value is a valid value */
         return (git_otype)value;
@@ -194,5 +274,20 @@ py_object_to_otype(PyObject *py_type)
         return py_type_to_git_type((PyTypeObject *) py_type);
 
     PyErr_SetString(PyExc_ValueError, "invalid target type");
-    return GIT_OBJ_BAD; /* -1 */
+    return GIT_OBJECT_INVALID; /* -1 */
+}
+
+
+/**
+ * Convert an integer to a reference to an IntEnum or IntFlag in pygit2.enums.
+ */
+PyObject *
+pygit2_enum(PyObject *enum_type, int value)
+{
+    if (!enum_type) {
+        PyErr_SetString(PyExc_TypeError, "an enum has not been cached in _pygit2.cache_enums()");
+        return NULL;
+    }
+    PyObject *enum_instance = PyObject_CallFunction(enum_type, "(i)", value);
+    return enum_instance;
 }
