@@ -1,4 +1,4 @@
-# Copyright 2010-2021 The pygit2 contributors
+# Copyright 2010-2024 The pygit2 contributors
 #
 # This file is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License, version 2,
@@ -27,7 +27,8 @@
 
 import pytest
 
-from pygit2 import Signature, Oid, GIT_BLAME_IGNORE_WHITESPACE
+from pygit2 import Signature, Oid
+from pygit2.enums import BlameFlag
 
 
 PATH = 'hello.txt'
@@ -63,7 +64,7 @@ def test_blame_index(testrepo):
 
 
 def test_blame_flags(blameflagsrepo):
-    blame = blameflagsrepo.blame(PATH, flags=GIT_BLAME_IGNORE_WHITESPACE)
+    blame = blameflagsrepo.blame(PATH, flags=BlameFlag.IGNORE_WHITESPACE)
 
     assert len(blame) == 3
 
