@@ -1,4 +1,4 @@
-# Copyright 2010-2021 The pygit2 contributors
+# Copyright 2010-2024 The pygit2 contributors
 #
 # This file is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License, version 2,
@@ -27,10 +27,13 @@
 
 import io
 from pathlib import Path
+from threading import Event
+from queue import Queue
 
 import pytest
 
 import pygit2
+from pygit2.enums import ObjectType
 from . import utils
 
 
@@ -84,7 +87,7 @@ def test_read_blob(testrepo):
     assert sha == BLOB_SHA
     assert isinstance(blob, pygit2.Blob)
     assert not blob.is_binary
-    assert pygit2.GIT_OBJ_BLOB == blob.type
+    assert ObjectType.BLOB == blob.type
     assert BLOB_CONTENT == blob.data
     assert len(BLOB_CONTENT) == blob.size
     assert BLOB_CONTENT == blob.read_raw()
@@ -94,7 +97,7 @@ def test_create_blob(testrepo):
     blob = testrepo[blob_oid]
 
     assert isinstance(blob, pygit2.Blob)
-    assert pygit2.GIT_OBJ_BLOB == blob.type
+    assert ObjectType.BLOB == blob.type
 
     assert blob_oid == blob.id
     assert utils.gen_blob_sha1(BLOB_NEW_CONTENT) == blob_oid.hex
@@ -115,7 +118,7 @@ def test_create_blob_fromworkdir(testrepo):
     blob = testrepo[blob_oid]
 
     assert isinstance(blob, pygit2.Blob)
-    assert pygit2.GIT_OBJ_BLOB == blob.type
+    assert ObjectType.BLOB == blob.type
 
     assert blob_oid == blob.id
     assert utils.gen_blob_sha1(BLOB_FILE_CONTENT) == blob_oid.hex
@@ -140,7 +143,7 @@ def test_create_blob_fromdisk(testrepo):
     blob = testrepo[blob_oid]
 
     assert isinstance(blob, pygit2.Blob)
-    assert pygit2.GIT_OBJ_BLOB == blob.type
+    assert ObjectType.BLOB == blob.type
 
 def test_create_blob_fromiobase(testrepo):
     with pytest.raises(TypeError):
@@ -151,7 +154,7 @@ def test_create_blob_fromiobase(testrepo):
     blob = testrepo[blob_oid]
 
     assert isinstance(blob, pygit2.Blob)
-    assert pygit2.GIT_OBJ_BLOB == blob.type
+    assert ObjectType.BLOB == blob.type
 
     assert blob_oid == blob.id
     assert BLOB_SHA == blob_oid.hex
@@ -192,3 +195,44 @@ def test_blob_from_repo(testrepo):
     patch_two = blob.diff_to_buffer(None)
 
     assert patch_one.text == patch_two.text
+
+def test_blob_write_to_queue(testrepo):
+    queue = Queue()
+    ready = Event()
+    done = Event()
+    blob = testrepo[BLOB_SHA]
+    blob._write_to_queue(queue, ready, done)
+    assert ready.wait()
+    assert done.wait()
+    chunks = []
+    while not queue.empty():
+        chunks.append(queue.get())
+    assert BLOB_CONTENT == b''.join(chunks)
+
+def test_blob_write_to_queue_filtered(testrepo):
+    queue = Queue()
+    ready = Event()
+    done = Event()
+    blob_oid = testrepo.create_blob_fromworkdir('bye.txt')
+    blob = testrepo[blob_oid]
+    blob._write_to_queue(queue, ready, done, as_path='bye.txt')
+    assert ready.wait()
+    assert done.wait()
+    chunks = []
+    while not queue.empty():
+        chunks.append(queue.get())
+    assert b'bye world\n' == b''.join(chunks)
+
+def test_blobio(testrepo):
+    blob_oid = testrepo.create_blob_fromworkdir('bye.txt')
+    blob = testrepo[blob_oid]
+    with pygit2.BlobIO(blob) as reader:
+        assert b'bye world\n' == reader.read()
+    assert not reader.raw._thread.is_alive()
+
+def test_blobio_filtered(testrepo):
+    blob_oid = testrepo.create_blob_fromworkdir('bye.txt')
+    blob = testrepo[blob_oid]
+    with pygit2.BlobIO(blob, as_path='bye.txt') as reader:
+        assert b'bye world\n' == reader.read()
+    assert not reader.raw._thread.is_alive()
