@@ -1,4 +1,4 @@
-# Copyright 2010-2021 The pygit2 contributors
+# Copyright 2010-2024 The pygit2 contributors
 #
 # This file is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License, version 2,
@@ -27,7 +27,7 @@
 
 import pytest
 
-from pygit2 import GIT_DESCRIBE_TAGS, GIT_DESCRIBE_ALL
+from pygit2.enums import DescribeStrategy, ObjectType
 import pygit2
 
 
@@ -35,7 +35,7 @@ def add_tag(repo, name, target):
     message = 'Example tag.\n'
     tagger = pygit2.Signature('John Doe', 'jdoe@example.com', 12347, 0)
 
-    sha = repo.create_tag(name, target, pygit2.GIT_OBJ_COMMIT, tagger, message)
+    sha = repo.create_tag(name, target, ObjectType.COMMIT, tagger, message)
     return sha
 
 
@@ -50,11 +50,11 @@ def test_describe_default_oid(testrepo):
     assert '2be5719' == testrepo.describe(show_commit_oid_as_fallback=True)
 
 def test_describe_strategies(testrepo):
-    assert 'heads/master' == testrepo.describe(describe_strategy=GIT_DESCRIBE_ALL)
+    assert 'heads/master' == testrepo.describe(describe_strategy=DescribeStrategy.ALL)
 
     testrepo.create_reference('refs/tags/thetag', '4ec4389a8068641da2d6578db0419484972284c8')
     with pytest.raises(KeyError): testrepo.describe()
-    assert 'thetag-2-g2be5719' == testrepo.describe(describe_strategy=GIT_DESCRIBE_TAGS)
+    assert 'thetag-2-g2be5719' == testrepo.describe(describe_strategy=DescribeStrategy.TAGS)
 
 def test_describe_pattern(testrepo):
     add_tag(testrepo, 'private/tag1', '5ebeeebb320790caf276b9fc8b24546d63316533')
