@@ -1,4 +1,4 @@
-# Copyright 2010-2021 The pygit2 contributors
+# Copyright 2010-2024 The pygit2 contributors
 #
 # This file is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License, version 2,
@@ -29,9 +29,9 @@ from pathlib import Path
 
 import pytest
 
-from pygit2 import GIT_REF_OID, GIT_REF_SYMBOLIC, Signature
-from pygit2 import Commit, Tree, reference_is_valid_name
+from pygit2 import Commit, Signature, Tree, reference_is_valid_name
 from pygit2 import AlreadyExistsError, GitError, InvalidSpecError
+from pygit2.enums import ReferenceType
 
 
 LAST_COMMIT = '2be5719152d4f82c7302b1c0932d8e5f0a4a0e98'
@@ -88,7 +88,8 @@ def test_refs_set_sha_prefix(testrepo):
 
 def test_refs_get_type(testrepo):
     reference = testrepo.references.get('refs/heads/master')
-    assert reference.type == GIT_REF_OID
+    assert reference.type == ReferenceType.DIRECT
+    assert reference.type == ReferenceType.OID  # deprecated
 
 def test_refs_get_target(testrepo):
     reference = testrepo.references.get('HEAD')
@@ -142,12 +143,17 @@ def test_refs_delete(testrepo):
 
 def test_refs_rename(testrepo):
     # We add a tag as a new reference that points to "origin/master"
-    reference = testrepo.references.create('refs/tags/version1',
-                                            LAST_COMMIT)
+    reference = testrepo.references.create('refs/tags/version1', LAST_COMMIT)
     assert reference.name == 'refs/tags/version1'
     reference.rename('refs/tags/version2')
     assert reference.name == 'refs/tags/version2'
 
+    with pytest.raises(AlreadyExistsError):
+        reference.rename('refs/tags/version2')
+
+    with pytest.raises(InvalidSpecError):
+        reference.rename('b1')
+
 #def test_reload(testrepo):
 #    name = 'refs/tags/version1'
 #    ref = testrepo.create_reference(name, "refs/heads/master", symbolic=True)
@@ -160,9 +166,10 @@ def test_refs_rename(testrepo):
 
 def test_refs_resolve(testrepo):
     reference = testrepo.references.get('HEAD')
-    assert reference.type == GIT_REF_SYMBOLIC
+    assert reference.type == ReferenceType.SYMBOLIC
     reference = reference.resolve()
-    assert reference.type == GIT_REF_OID
+    assert reference.type == ReferenceType.DIRECT
+    assert reference.type == ReferenceType.OID  # deprecated
     assert reference.target.hex == LAST_COMMIT
 
 def test_refs_resolve_identity(testrepo):
@@ -192,7 +199,7 @@ def test_refs_create_symbolic(testrepo):
     # We add a tag as a new symbolic reference that always points to
     # "refs/heads/master"
     reference = testrepo.references.create('refs/tags/beta', 'refs/heads/master')
-    assert reference.type == GIT_REF_SYMBOLIC
+    assert reference.type == ReferenceType.SYMBOLIC
     assert reference.target == 'refs/heads/master'
     assert reference.raw_target == b'refs/heads/master'
 
@@ -203,7 +210,7 @@ def test_refs_create_symbolic(testrepo):
     # try to create existing symbolic reference with force
     reference = testrepo.references.create('refs/tags/beta', 'refs/heads/master',
                                            force=True)
-    assert reference.type == GIT_REF_SYMBOLIC
+    assert reference.type == ReferenceType.SYMBOLIC
     assert reference.target == 'refs/heads/master'
     assert reference.raw_target == b'refs/heads/master'
 
@@ -272,6 +279,102 @@ def test_list_all_references(testrepo):
                                                      b'refs/heads/master',
                                                      b'refs/tags/version1']
 
+def test_references_iterator_init(testrepo):
+    repo = testrepo
+    iter = repo.references_iterator_init()
+
+    assert iter.__class__.__name__ ==  'RefsIterator'
+
+def test_references_iterator_next(testrepo):
+    repo = testrepo
+    repo.create_reference('refs/tags/version1', "2be5719152d4f82c7302b1c0932d8e5f0a4a0e98")
+    repo.create_reference('refs/tags/version2', "2be5719152d4f82c7302b1c0932d8e5f0a4a0e98")
+
+    iter_all = repo.references_iterator_init()
+    all_refs = []
+    for _ in range(4):
+        curr_ref = repo.references_iterator_next(iter_all)
+        if curr_ref:
+            all_refs.append((curr_ref.name, curr_ref.target.hex))
+
+    assert sorted(all_refs) == [
+        ('refs/heads/i18n', '5470a671a80ac3789f1a6a8cefbcf43ce7af0563'),
+        ('refs/heads/master', '2be5719152d4f82c7302b1c0932d8e5f0a4a0e98'),
+        ('refs/tags/version1', '2be5719152d4f82c7302b1c0932d8e5f0a4a0e98'),
+        ('refs/tags/version2', '2be5719152d4f82c7302b1c0932d8e5f0a4a0e98')]
+
+    iter_branches = repo.references_iterator_init()
+    all_branches = []
+    for _ in range(4):
+        curr_ref = repo.references_iterator_next(iter_branches, 1)
+        if curr_ref:
+            all_branches.append((curr_ref.name, curr_ref.target.hex))
+
+    assert sorted(all_branches) == [
+        ('refs/heads/i18n', '5470a671a80ac3789f1a6a8cefbcf43ce7af0563'),
+        ('refs/heads/master', '2be5719152d4f82c7302b1c0932d8e5f0a4a0e98')]
+
+    iter_tags = repo.references_iterator_init()
+    all_tags = []
+    for _ in range(4):
+        curr_ref = repo.references_iterator_next(iter_tags, 2)
+        if curr_ref:
+            all_tags.append((curr_ref.name, curr_ref.target.hex))
+
+    assert sorted(all_tags) == [
+        ('refs/tags/version1', '2be5719152d4f82c7302b1c0932d8e5f0a4a0e98'),
+        ('refs/tags/version2', '2be5719152d4f82c7302b1c0932d8e5f0a4a0e98')]
+
+def test_references_iterator_next_python(testrepo):
+    repo = testrepo
+    repo.create_reference('refs/tags/version1', "2be5719152d4f82c7302b1c0932d8e5f0a4a0e98")
+    repo.create_reference('refs/tags/version2', "2be5719152d4f82c7302b1c0932d8e5f0a4a0e98")
+
+    refs = []
+    for ref in repo.references.iterator():
+        refs.append((ref.name, ref.target.hex))
+
+    assert sorted(refs) == [
+        ('refs/heads/i18n', '5470a671a80ac3789f1a6a8cefbcf43ce7af0563'),
+        ('refs/heads/master', '2be5719152d4f82c7302b1c0932d8e5f0a4a0e98'),
+        ('refs/tags/version1', '2be5719152d4f82c7302b1c0932d8e5f0a4a0e98'),
+        ('refs/tags/version2', '2be5719152d4f82c7302b1c0932d8e5f0a4a0e98')]
+
+    branches = []
+    for branch in repo.references.iterator(1):
+        branches.append((branch.name, branch.target.hex))
+
+    assert sorted(branches) == [
+        ('refs/heads/i18n', '5470a671a80ac3789f1a6a8cefbcf43ce7af0563'),
+        ('refs/heads/master', '2be5719152d4f82c7302b1c0932d8e5f0a4a0e98')]
+
+    tags = []
+    for tag in repo.references.iterator(2):
+        tags.append((tag.name, tag.target.hex))
+
+    assert sorted(tags) == [
+        ('refs/tags/version1', '2be5719152d4f82c7302b1c0932d8e5f0a4a0e98'),
+        ('refs/tags/version2', '2be5719152d4f82c7302b1c0932d8e5f0a4a0e98')]
+
+def test_references_iterator_invalid_filter(testrepo):
+    repo = testrepo
+    iter_all = repo.references_iterator_init()
+
+    all_refs = []
+    for _ in range(4):
+        curr_ref = repo.references_iterator_next(iter_all, 5)
+        if curr_ref:
+            all_refs.append((curr_ref.name, curr_ref.target.hex))
+
+    assert all_refs == []
+
+def test_references_iterator_invalid_filter_python(testrepo):
+    repo = testrepo
+    refs = []
+    with pytest.raises(ValueError):
+        for ref in repo.references.iterator(5):
+            refs.append((ref.name, ref.target.hex))
+
 def test_lookup_reference(testrepo):
     repo = testrepo
 
@@ -367,7 +470,8 @@ def test_reference_set_sha_prefix(testrepo):
 
 def test_reference_get_type(testrepo):
     reference = testrepo.lookup_reference('refs/heads/master')
-    assert reference.type == GIT_REF_OID
+    assert reference.type == ReferenceType.DIRECT
+    assert reference.type == ReferenceType.OID  # deprecated
 
 def test_get_target(testrepo):
     reference = testrepo.lookup_reference('HEAD')
@@ -445,9 +549,10 @@ def test_rename(testrepo):
 
 def test_reference_resolve(testrepo):
     reference = testrepo.lookup_reference('HEAD')
-    assert reference.type == GIT_REF_SYMBOLIC
+    assert reference.type == ReferenceType.SYMBOLIC
     reference = reference.resolve()
-    assert reference.type == GIT_REF_OID
+    assert reference.type == ReferenceType.DIRECT
+    assert reference.type == ReferenceType.OID  # deprecated
     assert reference.target.hex == LAST_COMMIT
 
 def test_reference_resolve_identity(testrepo):
@@ -494,7 +599,7 @@ def test_create_symbolic_reference(testrepo):
     # "refs/heads/master"
     reference = repo.create_reference('refs/tags/beta',
                                       'refs/heads/master')
-    assert reference.type == GIT_REF_SYMBOLIC
+    assert reference.type == ReferenceType.SYMBOLIC
     assert reference.target == 'refs/heads/master'
     assert reference.raw_target == b'refs/heads/master'
 
@@ -506,7 +611,7 @@ def test_create_symbolic_reference(testrepo):
     # try to create existing symbolic reference with force
     reference = repo.create_reference('refs/tags/beta',
                                       'refs/heads/master', force=True)
-    assert reference.type == GIT_REF_SYMBOLIC
+    assert reference.type == ReferenceType.SYMBOLIC
     assert reference.target == 'refs/heads/master'
     assert reference.raw_target == b'refs/heads/master'
 
