@@ -1,4 +1,4 @@
-# Copyright 2010-2021 The pygit2 contributors
+# Copyright 2010-2024 The pygit2 contributors
 #
 # This file is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License, version 2,
@@ -23,12 +23,10 @@
 # the Free Software Foundation, 51 Franklin Street, Fifth Floor,
 # Boston, MA 02110-1301, USA.
 
-"""Tests for Repository objects."""
-
-# Standard Library
+import os
+from pathlib import Path
 import shutil
 import tempfile
-from pathlib import Path
 
 import pytest
 
@@ -36,6 +34,15 @@ import pytest
 import pygit2
 from pygit2 import init_repository, clone_repository, discover_repository
 from pygit2 import Oid
+from pygit2.enums import (
+    CheckoutNotify,
+    CheckoutStrategy,
+    FileStatus,
+    ObjectType,
+    RepositoryOpenFlag,
+    ResetMode,
+    StashApplyProgress,
+)
 from . import utils
 
 
@@ -69,7 +76,7 @@ def test_checkout_ref(testrepo):
     head = testrepo.head
     head = testrepo[head.target]
     assert 'new' not in head.tree
-    testrepo.checkout(ref_i18n, strategy=pygit2.GIT_CHECKOUT_FORCE)
+    testrepo.checkout(ref_i18n, strategy=CheckoutStrategy.FORCE)
 
     head = testrepo.head
     head = testrepo[head.target]
@@ -86,13 +93,13 @@ def test_checkout_callbacks(testrepo):
             self.conflicting_paths = set()
             self.updated_paths = set()
 
-        def checkout_notify_flags(self) -> int:
-            return pygit2.GIT_CHECKOUT_NOTIFY_CONFLICT | pygit2.GIT_CHECKOUT_NOTIFY_UPDATED
+        def checkout_notify_flags(self) -> CheckoutNotify:
+            return CheckoutNotify.CONFLICT | CheckoutNotify.UPDATED
 
         def checkout_notify(self, why, path, baseline, target, workdir):
-            if why == pygit2.GIT_CHECKOUT_NOTIFY_CONFLICT:
+            if why == CheckoutNotify.CONFLICT:
                 self.conflicting_paths.add(path)
-            elif why == pygit2.GIT_CHECKOUT_NOTIFY_UPDATED:
+            elif why == CheckoutNotify.UPDATED:
                 self.updated_paths.add(path)
 
     # checkout i18n with conflicts and default strategy should not be possible
@@ -106,7 +113,7 @@ def test_checkout_callbacks(testrepo):
     head = testrepo[head.target]
     assert 'new' not in head.tree
     callbacks = MyCheckoutCallbacks()
-    testrepo.checkout(ref_i18n, strategy=pygit2.GIT_CHECKOUT_FORCE, callbacks=callbacks)
+    testrepo.checkout(ref_i18n, strategy=CheckoutStrategy.FORCE, callbacks=callbacks)
     # make sure the callbacks caught the files affected by the checkout
     assert set() == callbacks.conflicting_paths
     assert {'bye.txt', 'new'} == callbacks.updated_paths
@@ -118,7 +125,7 @@ def test_checkout_aborted_from_callbacks(testrepo):
         return testrepo[testrepo.create_blob_fromworkdir("bye.txt")].data
 
     s = testrepo.status()
-    assert s == {'bye.txt': pygit2.GIT_STATUS_WT_NEW}
+    assert s == {'bye.txt': FileStatus.WT_NEW}
 
     class MyCheckoutCallbacks(pygit2.CheckoutCallbacks):
         def __init__(self):
@@ -140,7 +147,7 @@ def test_checkout_aborted_from_callbacks(testrepo):
 
     # checkout i18n with GIT_CHECKOUT_FORCE - callbacks should prevent checkout from completing
     with pytest.raises(InterruptedError):
-        testrepo.checkout(ref_i18n, strategy=pygit2.GIT_CHECKOUT_FORCE, callbacks=callbacks)
+        testrepo.checkout(ref_i18n, strategy=CheckoutStrategy.FORCE, callbacks=callbacks)
 
     assert callbacks.invoked_times == 2
     assert 'new' not in head.tree
@@ -157,7 +164,7 @@ def test_checkout_branch(testrepo):
     head = testrepo.head
     head = testrepo[head.target]
     assert 'new' not in head.tree
-    testrepo.checkout(branch_i18n, strategy=pygit2.GIT_CHECKOUT_FORCE)
+    testrepo.checkout(branch_i18n, strategy=CheckoutStrategy.FORCE)
 
     head = testrepo.head
     head = testrepo[head.target]
@@ -172,7 +179,7 @@ def test_checkout_index(testrepo):
 
     # checkout index
     assert 'hello.txt' in testrepo.status()
-    testrepo.checkout(strategy=pygit2.GIT_CHECKOUT_FORCE)
+    testrepo.checkout(strategy=CheckoutStrategy.FORCE)
     assert 'hello.txt' not in testrepo.status()
 
 def test_checkout_head(testrepo):
@@ -183,11 +190,11 @@ def test_checkout_head(testrepo):
 
     # checkout from index should not change anything
     assert 'bye.txt' in testrepo.status()
-    testrepo.checkout(strategy=pygit2.GIT_CHECKOUT_FORCE)
+    testrepo.checkout(strategy=CheckoutStrategy.FORCE)
     assert 'bye.txt' in testrepo.status()
 
     # checkout from head will reset index as well
-    testrepo.checkout('HEAD', strategy=pygit2.GIT_CHECKOUT_FORCE)
+    testrepo.checkout('HEAD', strategy=CheckoutStrategy.FORCE)
     assert 'bye.txt' not in testrepo.status()
 
 def test_checkout_alternative_dir(testrepo):
@@ -204,7 +211,7 @@ def test_checkout_paths(testrepo):
     testrepo.checkout(ref_master)
     testrepo.checkout(ref_i18n, paths=['new'])
     status = testrepo.status()
-    assert status['new'] == pygit2.GIT_STATUS_INDEX_NEW
+    assert status['new'] == FileStatus.INDEX_NEW
 
 def test_merge_base(testrepo):
     commit = testrepo.merge_base(
@@ -258,9 +265,7 @@ def test_reset_hard(testrepo):
     assert "hola mundo\n" in lines
     assert "bonjour le monde\n" in lines
 
-    testrepo.reset(
-        ref,
-        pygit2.GIT_RESET_HARD)
+    testrepo.reset(ref, ResetMode.HARD)
     assert testrepo.head.target.hex == ref
 
     with (Path(testrepo.workdir) / "hello.txt").open() as f:
@@ -276,9 +281,7 @@ def test_reset_soft(testrepo):
     assert "hola mundo\n" in lines
     assert "bonjour le monde\n" in lines
 
-    testrepo.reset(
-        ref,
-        pygit2.GIT_RESET_SOFT)
+    testrepo.reset(ref, ResetMode.SOFT)
     assert testrepo.head.target.hex == ref
     with (Path(testrepo.workdir) / "hello.txt").open() as f:
         lines = f.readlines()
@@ -297,9 +300,7 @@ def test_reset_mixed(testrepo):
     assert "hola mundo\n" in lines
     assert "bonjour le monde\n" in lines
 
-    testrepo.reset(
-        ref,
-        pygit2.GIT_RESET_MIXED)
+    testrepo.reset(ref, ResetMode.MIXED)
 
     assert testrepo.head.target.hex == ref
 
@@ -348,6 +349,39 @@ def test_stash(testrepo):
 
     with pytest.raises(KeyError): testrepo.stash_pop()
 
+def test_stash_partial(testrepo):
+    stash_message = "custom stash message"
+    sig = pygit2.Signature(name='Stasher', email='stasher@example.com', time=1641000000, offset=0)
+
+    # make sure we're starting with no stashes
+    assert [] == testrepo.listall_stashes()
+
+    # some changes to working dir
+    with (Path(testrepo.workdir) / 'hello.txt').open('w') as f:
+        f.write('stash me')
+    with (Path(testrepo.workdir) / 'untracked2.txt').open('w') as f:
+        f.write('do not stash me')
+
+    assert testrepo.status()['hello.txt'] == FileStatus.WT_MODIFIED
+    assert testrepo.status()['bye.txt'] == FileStatus.WT_NEW
+    assert testrepo.status()['untracked2.txt'] == FileStatus.WT_NEW
+
+    def stash_pathspecs(paths):
+        stash_id = testrepo.stash(sig, message=stash_message, keep_all=True, paths=paths)
+        stash_commit = testrepo[stash_id].peel(pygit2.Commit)
+        stash_diff = testrepo.diff(stash_commit.parents[0], stash_commit)
+        stash_files = set(patch.delta.new_file.path for patch in stash_diff)
+        return stash_files == set(paths)
+
+    # Stash a modified file
+    assert stash_pathspecs(['hello.txt'])
+
+    # Stash one of several untracked files
+    assert stash_pathspecs(['bye.txt'])
+
+    # Stash a modified file and an untracked file
+    assert stash_pathspecs(['hello.txt', 'bye.txt'])
+
 def test_stash_progress_callback(testrepo):
     sig = pygit2.Signature(name='Stasher', email='stasher@example.com', time=1641000000, offset=0)
 
@@ -361,7 +395,7 @@ def test_stash_progress_callback(testrepo):
     progress_sequence = []
 
     class MyStashApplyCallbacks(pygit2.StashApplyCallbacks):
-        def stash_apply_progress(self, progress: int):
+        def stash_apply_progress(self, progress: StashApplyProgress):
             progress_sequence.append(progress)
 
     # apply the stash
@@ -369,13 +403,13 @@ def test_stash_progress_callback(testrepo):
 
     # make sure the callbacks were notified of all the steps
     assert progress_sequence == [
-        pygit2.GIT_STASH_APPLY_PROGRESS_LOADING_STASH,
-        pygit2.GIT_STASH_APPLY_PROGRESS_ANALYZE_INDEX,
-        pygit2.GIT_STASH_APPLY_PROGRESS_ANALYZE_MODIFIED,
-        pygit2.GIT_STASH_APPLY_PROGRESS_ANALYZE_UNTRACKED,
-        pygit2.GIT_STASH_APPLY_PROGRESS_CHECKOUT_UNTRACKED,
-        pygit2.GIT_STASH_APPLY_PROGRESS_CHECKOUT_MODIFIED,
-        pygit2.GIT_STASH_APPLY_PROGRESS_DONE,
+        StashApplyProgress.LOADING_STASH,
+        StashApplyProgress.ANALYZE_INDEX,
+        StashApplyProgress.ANALYZE_MODIFIED,
+        StashApplyProgress.ANALYZE_UNTRACKED,
+        StashApplyProgress.CHECKOUT_UNTRACKED,
+        StashApplyProgress.CHECKOUT_MODIFIED,
+        StashApplyProgress.DONE,
     ]
 
 def test_stash_aborted_from_callbacks(testrepo):
@@ -393,8 +427,8 @@ def test_stash_aborted_from_callbacks(testrepo):
     # define callbacks that will abort the unstash process
     # just as libgit2 is ready to write the files to disk
     class MyStashApplyCallbacks(pygit2.StashApplyCallbacks):
-        def stash_apply_progress(self, progress: int):
-            if progress == pygit2.GIT_STASH_APPLY_PROGRESS_CHECKOUT_UNTRACKED:
+        def stash_apply_progress(self, progress: StashApplyProgress):
+            if progress == StashApplyProgress.CHECKOUT_UNTRACKED:
                 raise InterruptedError("Stop applying the stash!")
 
     # attempt to apply and delete the stash; the callbacks will interrupt that
@@ -429,7 +463,7 @@ def test_stash_apply_checkout_options(testrepo):
     # define callbacks that raise an InterruptedError when checkout detects a conflict
     class MyStashApplyCallbacks(pygit2.StashApplyCallbacks):
         def checkout_notify(self, why, path, baseline, target, workdir):
-            if why == pygit2.GIT_CHECKOUT_NOTIFY_CONFLICT:
+            if why == CheckoutNotify.CONFLICT:
                 raise InterruptedError("Applying the stash would create a conflict")
 
     # overwrite hello.txt so that applying the stash would create a conflict
@@ -439,13 +473,13 @@ def test_stash_apply_checkout_options(testrepo):
     # apply the stash with the default (safe) strategy;
     # the callbacks should detect a conflict on checkout
     with pytest.raises(InterruptedError):
-        testrepo.stash_apply(strategy=pygit2.GIT_CHECKOUT_SAFE, callbacks=MyStashApplyCallbacks())
+        testrepo.stash_apply(strategy=CheckoutStrategy.SAFE, callbacks=MyStashApplyCallbacks())
 
     # hello.txt should be intact
     with hello_txt.open('r') as f: assert f.read() == 'conflicting content'
 
     # force apply the stash; this should work
-    testrepo.stash_apply(strategy=pygit2.GIT_CHECKOUT_FORCE, callbacks=MyStashApplyCallbacks())
+    testrepo.stash_apply(strategy=CheckoutStrategy.FORCE, callbacks=MyStashApplyCallbacks())
     with hello_txt.open('r') as f: assert f.read() == 'stashed content'
 
 
@@ -478,7 +512,7 @@ def test_default_signature(testrepo):
 def test_new_repo(tmp_path):
     repo = init_repository(tmp_path, False)
 
-    oid = repo.write(pygit2.GIT_OBJ_BLOB, "Test")
+    oid = repo.write(ObjectType.BLOB, "Test")
     assert type(oid) == Oid
 
     assert (tmp_path / '.git').exists()
@@ -547,6 +581,15 @@ def test_clone_bare_repository(barerepo, tmp_path):
     assert not repo.is_empty
     assert repo.is_bare
 
+@utils.requires_network
+def test_clone_shallow_repository(tmp_path):
+    # shallow cloning currently only works with remote repositories
+    url = 'https://github.com/libgit2/TestGitRepository'
+    repo = clone_repository(url, tmp_path / 'clone-shallow', depth=1)
+    assert not repo.is_empty
+    assert repo.is_shallow
+
+
 def test_clone_repository_and_remote_callbacks(barerepo, tmp_path):
     url = Path(barerepo.path).resolve().as_uri()
     repo_path = tmp_path / 'clone-into'
@@ -733,20 +776,20 @@ def test_open_extended(tmp_path):
         assert repo.workdir == orig_repo.workdir
 
         with pytest.raises(pygit2.GitError):
-            repo = pygit2.Repository(subdir_path, pygit2.GIT_REPOSITORY_OPEN_NO_SEARCH)
+            repo = pygit2.Repository(subdir_path, RepositoryOpenFlag.NO_SEARCH)
 
         # GIT_REPOSITORY_OPEN_NO_DOTGIT
         gitdir_path = path / '.git'
         with pytest.raises(pygit2.GitError):
-            repo = pygit2.Repository(path, pygit2.GIT_REPOSITORY_OPEN_NO_DOTGIT)
+            repo = pygit2.Repository(path, RepositoryOpenFlag.NO_DOTGIT)
 
-        repo = pygit2.Repository(gitdir_path, pygit2.GIT_REPOSITORY_OPEN_NO_DOTGIT)
+        repo = pygit2.Repository(gitdir_path, RepositoryOpenFlag.NO_DOTGIT)
         assert not repo.is_bare
         assert repo.path == orig_repo.path
         assert repo.workdir == orig_repo.workdir
 
         # GIT_REPOSITORY_OPEN_BARE
-        repo = pygit2.Repository(gitdir_path, pygit2.GIT_REPOSITORY_OPEN_BARE)
+        repo = pygit2.Repository(gitdir_path, RepositoryOpenFlag.BARE)
         assert repo.is_bare
         assert repo.path == orig_repo.path
         assert not repo.workdir
