@@ -1,4 +1,4 @@
-# Copyright 2010-2021 The pygit2 contributors
+# Copyright 2010-2024 The pygit2 contributors
 #
 # This file is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License, version 2,
@@ -22,9 +22,12 @@
 # along with this program; see the file COPYING.  If not, write to
 # the Free Software Foundation, 51 Franklin Street, Fifth Floor,
 # Boston, MA 02110-1301, USA.
-import pygit2
+
 import pytest
 
+import pygit2
+from pygit2.enums import FileStatus
+
 
 def test_status(dirtyrepo):
     """
@@ -38,7 +41,7 @@ def test_status(dirtyrepo):
 
 def test_status_untracked_no(dirtyrepo):
     git_status = dirtyrepo.status(untracked_files="no")
-    not any(status & pygit2.GIT_STATUS_WT_NEW for status in git_status.values())
+    assert not any(status & FileStatus.WT_NEW for status in git_status.values())
 
 
 @pytest.mark.parametrize(
@@ -68,7 +71,7 @@ def test_status_untracked_no(dirtyrepo):
 def test_status_untracked_normal(dirtyrepo, untracked_files, expected):
     git_status = dirtyrepo.status(untracked_files=untracked_files)
     assert {
-        file for file, status in git_status.items() if status & pygit2.GIT_STATUS_WT_NEW
+        file for file, status in git_status.items() if status & FileStatus.WT_NEW
     } == expected
 
 
@@ -78,5 +81,5 @@ def test_status_ignored(dirtyrepo, ignored, expected):
     assert {
         file
         for file, status in git_status.items()
-        if status & pygit2.GIT_STATUS_IGNORED
+        if status & FileStatus.IGNORED
     } == expected
