@@ -1,4 +1,4 @@
-# Copyright 2010-2021 The pygit2 contributors
+# Copyright 2010-2024 The pygit2 contributors
 #
 # This file is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License, version 2,
@@ -61,7 +61,7 @@ requires_proxy = pytest.mark.skipif(
     reason='Requires proxy at port 8888')
 
 requires_ssh = pytest.mark.skipif(
-    not (pygit2.features & pygit2.GIT_FEATURE_SSH),
+    pygit2.enums.Feature.SSH not in pygit2.features,
     reason='Requires SSH')
 
 
@@ -85,7 +85,7 @@ def force_rm_handle(remove_path, path, excinfo):
 def gen_blob_sha1(data):
     # http://stackoverflow.com/questions/552659/assigning-git-sha1s-without-git
     m = hashlib.sha1()
-    m.update(('blob %d\0' % len(data)).encode())
+    m.update(f'blob {len(data)}\0'.encode())
     m.update(data)
     return m.hexdigest()
 
